///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  Notice to licensees:                                                     //
//                                                                           //
//  This source code is the exclusive, proprietary intellectual property of  //
//  Sharkysoft (sharkysoft.com).  You may view this source code as a         //
//  supplement to other product documentation, but you may not distribute    //
//  it or use it for any other purpose without written consent from          //
//  Sharkysoft.                                                              //
//                                                                           //
//  You are permitted to modify and recompile this source code, but you may  //
//  not remove this notice.  If you add features to or fix errors in this    //
//  code, please consider sharing your changes with Sharkysoft for possible  //
//  incorporation into future releases of the product.  Thanks!              //
//                                                                           //
//  For more information about Sharkysoft products and services, please      //
//  visit Sharkysoft on the web at                                           //
//                                                                           //
//       http://sharkysoft.com/                                              //
//                                                                           //
//  Thank you for using Lava!                                                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



package lava.array;



import java.lang.reflect.Array;
import lava.math.MathToolbox;



/******************************************************************************
Shifts and rotates arrays.

<p><b>Details:</b>  <code>ArrayShifter</code> is a collection of functions for shifting and rotating arrays.</p>

<p>Depending on the function name, the array is either manipulated in place, or the array is copied and then manipulated, leaving the original unmodified.  In all cases, the manipulated result is returned.</p>

<p>Negative amounts for rotating and shifting indicate shifting towards the beginning (zero element) of the array, and positive values shift towards the end.  Shifting by an amount greater than the length of the array produces an array filled with default values.  Rotating by an amount greater than the length of the array produces the result you would expect to get if the array were repeatedly rotated by one element that many times.</p>

<p>Functions in this class are not thread-safe.  If the input array is modified by another thread while it is being copied, shifted, or rotated, the results may be unpredictable.  Users should develop their own thread-safety protocol for arrays that may be affected in this manner.</p>

@version 1998.11.18
@since 1998.10.31
******************************************************************************/

public class ArrayShifter
{



	////////////////////////////////
	//                            //
	//  the rotateCopy functions  //
	//                            //
	////////////////////////////////



	/**********************************************************************
	Rotates an array of <code>double</code>s.

	@param ar the array
	@param amt amount of rotation
	@return the rotated array

	@version 1998.10.31
	@since 1988.10.31
	**********************************************************************/

	public final static double[] rotateCopy (double[] ar, int amt)
	{
		int len = ar . length;
		amt = MathToolbox.mod (- amt, 0, len);

		// Perform negative rotation by amt:
		double[] nar = new double [len];
		System.arraycopy (ar, amt, nar, 0, len - amt);
		System.arraycopy (ar, 0, nar, len - amt, amt);
		return nar;
	}



	/**********************************************************************
	Rotates an array of <code>float</code>s.

	@param ar the array
	@param amt amount of rotation
	@return the rotated array

	@version 1998.10.31
	@since 1988.10.31
	**********************************************************************/

	public final static float[] rotateCopy (float[] ar, int amt)
	{
		int len = ar . length;
		amt = MathToolbox.mod (- amt, 0, len);

		// Perform negative rotation by amt:
		float[] nar = new float [len];
		System.arraycopy (ar, amt, nar, 0, len - amt);
		System.arraycopy (ar, 0, nar, len - amt, amt);
		return nar;
	}



	/**********************************************************************
	Rotates an array of <code>long</code>s.

	@param ar the array
	@param amt amount of rotation
	@return the rotated array

	@version 1998.10.31
	@since 1988.10.31
	**********************************************************************/

	public final static long[] rotateCopy (long[] ar, int amt)
	{
		int len = ar . length;
		amt = MathToolbox.mod (- amt, 0, len);

		// Perform negative rotation by amt:
		long[] nar = new long [len];
		System.arraycopy (ar, amt, nar, 0, len - amt);
		System.arraycopy (ar, 0, nar, len - amt, amt);
		return nar;
	}



	/**********************************************************************
	Rotates an array of <code>int</code>s.

	@param ar the array
	@param amt amount of rotation
	@return the rotated array

	@version 1998.10.31
	@since 1988.10.31
	**********************************************************************/

	public final static int[] rotateCopy (int[] ar, int amt)
	{
		int len = ar . length;
		amt = MathToolbox.mod (- amt, 0, len);

		// Perform negative rotation by amt:
		int[] nar = new int [len];
		System.arraycopy (ar, amt, nar, 0, len - amt);
		System.arraycopy (ar, 0, nar, len - amt, amt);
		return nar;
	}



	/**********************************************************************
	Rotates an array of <code>short</code>s.

	@param ar the array
	@param amt amount of rotation
	@return the rotated array

	@version 1998.10.31
	@since 1988.10.31
	**********************************************************************/

	public final static short[] rotateCopy (short[] ar, int amt)
	{
		int len = ar . length;
		amt = MathToolbox.mod (- amt, 0, len);

		// Perform negative rotation by amt:
		short[] nar = new short [len];
		System.arraycopy (ar, amt, nar, 0, len - amt);
		System.arraycopy (ar, 0, nar, len - amt, amt);
		return nar;
	}



	/**********************************************************************
	Rotates an array of <code>byte</code>s.

	@param ar the array
	@param amt amount of rotation
	@return the rotated array

	@version 1998.10.31
	@since 1988.10.31
	**********************************************************************/

	public final static byte[] rotateCopy (byte[] ar, int amt)
	{
		int len = ar . length;
		amt = MathToolbox.mod (- amt, 0, len);

		// Perform negative rotation by amt:
		byte[] nar = new byte [len];
		System.arraycopy (ar, amt, nar, 0, len - amt);
		System.arraycopy (ar, 0, nar, len - amt, amt);
		return nar;
	}



	/**********************************************************************
	Rotates an array of <code>char</code>s.

	@param ar the array
	@param amt amount of rotation
	@return the rotated array

	@version 1998.10.31
	@since 1988.10.31
	**********************************************************************/

	public final static char[] rotateCopy (char[] ar, int amt)
	{
		int len = ar . length;
		amt = MathToolbox.mod (- amt, 0, len);

		// Perform negative rotation by amt:
		char[] nar = new char [len];
		System.arraycopy (ar, amt, nar, 0, len - amt);
		System.arraycopy (ar, 0, nar, len - amt, amt);
		return nar;
	}



	/**********************************************************************
	Rotates an array of <code>boolean</code>s.

	@param ar the array
	@param amt amount of rotation
	@return the rotated array

	@version 1998.10.31
	@since 1988.10.31
	**********************************************************************/

	public final static boolean[] rotateCopy (boolean[] ar, int amt)
	{
		int len = ar . length;
		amt = MathToolbox.mod (- amt, 0, len);

		// Perform negative rotation by amt:
		boolean[] nar = new boolean [len];
		System.arraycopy (ar, amt, nar, 0, len - amt);
		System.arraycopy (ar, 0, nar, len - amt, amt);
		return nar;
	}



	/**********************************************************************
	Rotates an array of <code>Object</code>s.

	<p><b>Details:</b> <code>shiftCopy</code> uses Java reflection to create a new array of the same type and length as the input, performs the rotate-copy, and returns the new array.  The result should probably be type-casted into the desired array type.</p>

	@param ar the array
	@param amt amount of rotation
	@return the rotated array

	@version 1998.10.31
	@since 1988.10.31
	**********************************************************************/

	public final static Object[] rotateCopy (Object[] ar, int amt)
	{
		int len = ar . length;
		amt = MathToolbox.mod (- amt, 0, len);

		// Perform negative rotation by amt:
		Object[] nar = (Object[]) Array.newInstance (ar . getClass () . getComponentType (), len);
		System.arraycopy (ar, amt, nar, 0, len - amt);
		System.arraycopy (ar, 0, nar, len - amt, amt);
		return nar;
	}



	///////////////////////////////
	//                           //
	//  the shiftCopy functions  //
	//                           //
	///////////////////////////////



	/**********************************************************************
	Shifts an array of <code>Object</code>s.

	<p><b>Details:</b> <code>shiftCopy</code> uses Java reflection to create a new array of the same type and length as the input, performs the shift-copy, and returns the new array.  The result should probably be type-casted into the desired array type.</p>

	@param ar the array
	@param amt amount of shift
	@return the shifted array

	@version 1998.10.31
	@since 1988.10.31
	**********************************************************************/

	public final static Object[] shiftCopy (Object[] ar, int amt)
	{
		int len = ar . length;
		Object[] nar = (Object[]) Array.newInstance (ar . getClass () . getComponentType (), len);
		if (amt >= 0)
		{
			// Positive shift:
			if (amt > len)
				amt = len;
			System.arraycopy (ar, 0, nar, amt, len - amt);
		}
		else if (amt < 0)
		{
			// Negative shift:
			amt = - amt;
			if (amt > len)
				amt = len;
			System.arraycopy (ar, amt, nar, 0, len - amt);
		}
		return nar;
	}



	/**********************************************************************
	Shifts an array of <code>double</code>s.

	@param ar the array
	@param amt amount of shift
	@return the shifted array

	@version 1998.10.31
	@since 1988.10.31
	**********************************************************************/

	public final static double[] shiftCopy (double[] ar, int amt)
	{
		int len = ar . length;
		double[] nar = new double [len];
		if (amt >= 0)
		{
			// Positive shift:
			if (amt > len)
				amt = len;
			System.arraycopy (ar, 0, nar, amt, len - amt);
		}
		else if (amt < 0)
		{
			// Negative shift:
			amt = - amt;
			if (amt > len)
				amt = len;
			System.arraycopy (ar, amt, nar, 0, len - amt);
		}
		return nar;
	}



	/**********************************************************************
	Shifts an array of <code>float</code>s.

	@param ar the array
	@param amt amount of shift
	@return the shifted array

	@version 1998.10.31
	@since 1988.10.31
	**********************************************************************/

	public final static float[] shiftCopy (float[] ar, int amt)
	{
		int len = ar . length;
		float[] nar = new float [len];
		if (amt >= 0)
		{
			// Positive shift:
			if (amt > len)
				amt = len;
			System.arraycopy (ar, 0, nar, amt, len - amt);
		}
		else if (amt < 0)
		{
			// Negative shift:
			amt = - amt;
			if (amt > len)
				amt = len;
			System.arraycopy (ar, amt, nar, 0, len - amt);
		}
		return nar;
	}



	/**********************************************************************
	Shifts an array of <code>long</code>s.

	@param ar the array
	@param amt amount of shift
	@return the shifted array

	@version 1998.10.31
	@since 1988.10.31
	**********************************************************************/

	public final static long[] shiftCopy (long[] ar, int amt)
	{
		int len = ar . length;
		long[] nar = new long [len];
		if (amt >= 0)
		{
			// Positive shift:
			if (amt > len)
				amt = len;
			System.arraycopy (ar, 0, nar, amt, len - amt);
		}
		else if (amt < 0)
		{
			// Negative shift:
			amt = - amt;
			if (amt > len)
				amt = len;
			System.arraycopy (ar, amt, nar, 0, len - amt);
		}
		return nar;
	}



	/**********************************************************************
	Shifts an array of <code>int</code>s.

	@param ar the array
	@param amt amount of shift
	@return the shifted array

	@version 1998.10.31
	@since 1988.10.31
	**********************************************************************/

	public final static int[] shiftCopy (int[] ar, int amt)
	{
		int len = ar . length;
		int[] nar = new int [len];
		if (amt >= 0)
		{
			// Positive shift:
			if (amt > len)
				amt = len;
			System.arraycopy (ar, 0, nar, amt, len - amt);
		}
		else if (amt < 0)
		{
			// Negative shift:
			amt = - amt;
			if (amt > len)
				amt = len;
			System.arraycopy (ar, amt, nar, 0, len - amt);
		}
		return nar;
	}



	/**********************************************************************
	Shifts an array of <code>short</code>s.

	@param ar the array
	@param amt amount of shift
	@return the shifted array

	@version 1998.10.31
	@since 1988.10.31
	**********************************************************************/

	public final static short[] shiftCopy (short[] ar, int amt)
	{
		int len = ar . length;
		short[] nar = new short [len];
		if (amt >= 0)
		{
			// Positive shift:
			if (amt > len)
				amt = len;
			System.arraycopy (ar, 0, nar, amt, len - amt);
		}
		else if (amt < 0)
		{
			// Negative shift:
			amt = - amt;
			if (amt > len)
				amt = len;
			System.arraycopy (ar, amt, nar, 0, len - amt);
		}
		return nar;
	}



	/**********************************************************************
	Shifts an array of <code>char</code>s.

	@param ar the array
	@param amt amount of shift
	@return the shifted array

	@version 1998.10.31
	@since 1988.10.31
	**********************************************************************/

	public final static char[] shiftCopy (char[] ar, int amt)
	{
		int len = ar . length;
		char[] nar = new char [len];
		if (amt >= 0)
		{
			// Positive shift:
			if (amt > len)
				amt = len;
			System.arraycopy (ar, 0, nar, amt, len - amt);
		}
		else if (amt < 0)
		{
			// Negative shift:
			amt = - amt;
			if (amt > len)
				amt = len;
			System.arraycopy (ar, amt, nar, 0, len - amt);
		}
		return nar;
	}



	/**********************************************************************
	Shifts an array of <code>byte</code>s.

	@param ar the array
	@param amt amount of shift
	@return the shifted array

	@version 1998.10.31
	@since 1988.10.31
	**********************************************************************/

	public final static byte[] shiftCopy (byte[] ar, int amt)
	{
		int len = ar . length;
		byte[] nar = new byte [len];
		if (amt >= 0)
		{
			// Positive shift:
			if (amt > len)
				amt = len;
			System.arraycopy (ar, 0, nar, amt, len - amt);
		}
		else if (amt < 0)
		{
			// Negative shift:
			amt = - amt;
			if (amt > len)
				amt = len;
			System.arraycopy (ar, amt, nar, 0, len - amt);
		}
		return nar;
	}



	/**********************************************************************
	Shifts an array of <code>boolean</code>s.

	@param ar the array
	@param amt amount of shift
	@return the shifted array

	@version 1998.10.31
	@since 1988.10.31
	**********************************************************************/

	public final static boolean[] shiftCopy (boolean[] ar, int amt)
	{
		int len = ar . length;
		boolean[] nar = new boolean [len];
		if (amt >= 0)
		{
			// Positive shift:
			if (amt > len)
				amt = len;
			System.arraycopy (ar, 0, nar, amt, len - amt);
		}
		else if (amt < 0)
		{
			// Negative shift:
			amt = - amt;
			if (amt > len)
				amt = len;
			System.arraycopy (ar, amt, nar, 0, len - amt);
		}
		return nar;
	}



}



