///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  Notice to licensees:                                                     //
//                                                                           //
//  This source code is the exclusive, proprietary intellectual property of  //
//  Sharkysoft (sharkysoft.com).  You may view this source code as a         //
//  supplement to other product documentation, but you may not distribute    //
//  it or use it for any other purpose without written consent from          //
//  Sharkysoft.                                                              //
//                                                                           //
//  You are permitted to modify and recompile this source code, but you may  //
//  not remove this notice.  If you add features to or fix errors in this    //
//  code, please consider sharing your changes with Sharkysoft for possible  //
//  incorporation into future releases of the product.  Thanks!              //
//                                                                           //
//  For more information about Sharkysoft products and services, please      //
//  visit Sharkysoft on the web at                                           //
//                                                                           //
//       http://sharkysoft.com/                                              //
//                                                                           //
//  Thank you for using Lava!                                                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



package lava.riff;



import java.io.IOException;



/******************************************************************************
Riff input source.

<p><b>Details:</b>  <code>IRiffInput</code> defines methods for accessing the individual BYTEs, WORDs, DWORDs, and chunks of a RIFF data source.  Since this interface represents a generic byte input source, the underlying data source may or may not be file based.  While <code>IRiffInput</code> is typically used for reading static RIFF files, it can also be used to access other data sources, such as streaming data sources or dynamically created "virtual files."</p>

<p><code>IRiffInput</code> implementations maintain a "read offset," or a value that indicates the position of the next element to be read.  Because this value is settable, it is possible that RIFF data consumers will request RIFF data in random order.  (The read offset is similar to the "file pointer" concept used in <code>java.io.RandomAccessFile</code>).  The offset of the first byte in an <code>IRiffInput</code> data source is 0.</p>

<p>Because <code>IRiffInput</code> allows clients to read byte sequences in random order, some implementations will need to buffer their underlying data sources if they do not support random access.  To facilitate efficient memory usage, <code>IRiffInput</code> defines a <code>free</code> method which clients can use to give hints about segments that are no longer needed.  Implementations should take advantage of these hints to produce efficient buffering, and clients should provide as many of these hints as possible.</p>

<form>
<p>Clip-source:<br>
<textarea rows=6 cols=80 onChange="this.form.reset();">
</textarea>
</form>

@since 1999.09.24
******************************************************************************/

public interface IRiffInput
{



	/**********************************************************************
	Returns file length.

	<p><b>Details:</b>  This method returns the length of the RIFF file, if it is known, or -1 otherwise.</p>

	@return the length
	**********************************************************************/

	long getLength () throws IOException;



	/**********************************************************************
	Returns read offset.

	<p><b>Details:</b>  This method returns the read offset, or the position of the next byte to be read.  The position reported may be greater than the length of the stream if it was previously set to be so.  (See {@link #setPosition(long) <code>setPosition</code>}.)</p>

	@return the position
	**********************************************************************/

	long getPosition () throws IOException;



	/**********************************************************************
	Sets read offset.

	<p><b>Details:</b>  This method sets the read offset, or the position of the next byte to be read, to <var>newpos</var>.  Setting the position to a value greater than the length of the file is legal, and will not cause an error unless a read is subsequently attempted from that position.</p>

	@param newpos the new position
	**********************************************************************/

	void setPosition (long newpos) throws IOException;



	/**********************************************************************
	Reads byte.

	<p><b>Details:</b>  This method reads and returns the next BYTE, i.e., the BYTE pointed to by the read offset.  The read offset is automatically incremented by 1.  The BYTE is zero-<wbr>extended into an <code>int</code> before it is returned.  If a BYTE could not be read because the end of the data source was reached, -1 is returned.</p>

	@return the byte
	**********************************************************************/

	int read () throws IOException;



	/**********************************************************************
	Reads BYTEs.

	<p><b>Details:</b>  This method reads a consecutive run of BYTEs from the data source, beginning with the BYTE at the current position.  The BYTEs are stored in <var>dest</var> beginning at the given offset (<var>offset</var>).  Up to <var>amount</var> BYTEs are read and stored, but the actual number of BYTEs read may be less if the end of the sequence is reached.  If BYTEs are read, the number of BYTEs read is returned, but if no BYTEs can be read, -1 is returned.  The read offset is automatically advanced by the number of BYTEs read.</p>

	@param dest the destination array
	@param offset offset into dest
	@param amount number of bytes to read
	@return actual number of bytes read
	**********************************************************************/

	int read (byte[] dest, int offset, int amount) throws IOException;



	/**********************************************************************
	Reads WORDs.

	<p><b>Details:</b>  This method reads a consecutive run of <em>little-endian</em> WORDs, or 16-bit integers, from the data source, beginning with the WORD at the current position.  The WORDs are stored in <var>dest</var> beginning at the given offset (<var>offset</var>).  Up to <var>amount</var> WORDs are read and stored, but the actual number of WORDs read may be less if the end of the sequence is reached.  If WORDs are read, the number of WORDs read is returned, but if no WORDs can be read, -1 is returned.  The read offset is automatically advanced by the number of BYTEs read.</p>

	@param dest the destination array
	@param offset offset into dest
	@param amount number of bytes to read
	@return actual number of bytes read
	**********************************************************************/

	int read (short[] dest, int offset, int amount) throws IOException;



	/**********************************************************************
	Reads DWORDs.

	<p><b>Details:</b>  This method reads a consecutive run of <em>little-endian</em> DWORDs, or 32-bit integers, from the data source, beginning with the DWORD at the current position.  The DWORDs are stored in <var>dest</var> beginning at the given offset (<var>offset</var>).  Up to <var>amount</var> DWORDs are read and stored, but the actual number of DWORDs read may be less if the end of the sequence is reached.  If DWORDs are read, the number of DWORDs read is returned, but if no DWORDs can be read, -1 is returned.  The read offset is automatically advanced by the number of BYTEs read.</p>

	@param dest the destination array
	@param offset offset into dest
	@param amount number of bytes to read
	@return actual number of bytes read
	**********************************************************************/

	int read (int[] dest, int offset, int amount) throws IOException;



	/**********************************************************************
	Unbuffers bytes.

	<p><b>Details:</b>  This method advises this implementation that the given range of bytes can be unbuffered because the consumer has no need to read them again.  Revisiting segments of this data source after they have been <code>free</code>d can cause an exception, return incorrect data, or simply cause undetected, unpredictable results.</p>

	@param offset the position of the forgettable segment
	@param amount the length of the forgettable segment
	**********************************************************************/

	void free (long offset, long amount) throws IOException;



	/**********************************************************************
	Closes input source.

	<p><b>Details:</b>  <code>close</code> closes this data source and releases associated resources.  Calling any of the other methods after <code>close</code> may produce unpredictable results or generate an exception.</p>
	**********************************************************************/

	void close () throws IOException;



}



