///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  Notice to licensees:                                                     //
//                                                                           //
//  This source code is the exclusive, proprietary intellectual property of  //
//  Sharkysoft (sharkysoft.com).  You may view this source code as a         //
//  supplement to other product documentation, but you may not distribute    //
//  it or use it for any other purpose without written consent from          //
//  Sharkysoft.                                                              //
//                                                                           //
//  You are permitted to modify and recompile this source code, but you may  //
//  not remove this notice.  If you add features to or fix errors in this    //
//  code, please consider sharing your changes with Sharkysoft for possible  //
//  incorporation into future releases of the product.  Thanks!              //
//                                                                           //
//  For more information about Sharkysoft products and services, please      //
//  visit Sharkysoft on the web at                                           //
//                                                                           //
//       http://sharkysoft.com/                                              //
//                                                                           //
//  Thank you for using Lava!                                                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



package lava.riff;



import java.io.IOException;



/******************************************************************************
Riff output sink.

<p><b>Details:</b>  <code>IRiffOutput</code> defines methods for writing BYTEs, WORDs, DWORDs, and chunks to a RIFF data file.  Since this interface represents a generic byte output sink, the underlying data sink may or may not be file based.  While <code>IRiffOutput</code> is typically used for writing static RIFF files, it can also be used to produce other data products, such as RIFF "streams" shipped over a network.</p>

<p><code>IRiffOutput</code> implementations maintain a "write offset," or a value that indicates the position of the next element to be written.  Because this value is settable, it is possible that RIFF data producers will produce RIFF data in random order.  (The write offset is similar to the "file pointer" concept used in <code>java.io.RandomAccessFile</code>).  The offset of the first byte in an <code>IRiffOutput</code> data source is 0.</p>

<p>Because <code>IRiffOutput</code> allows clients to write byte sequences in random order, some implementations will need to buffer their underlying data sinks if they do not support random access.  <code>IRiffOutput</code> maintains a "commit offset," or the offset of the first byte in the sequence whose value is still mutable.  In other words, all bytes written through this interface are subject to overwriting until the commit offset has advanced beyond them (through calls to <code>flush</code>).  To facilitate efficient memory usage and reduce buffering requirements, interface users should commit data as soon and as often as possible.</p>

@version 1999.09.24
@since 1999.09.24
******************************************************************************/

public interface IRiffOutput
{



	/**********************************************************************
	Sets stream length.

	<p><b>Details:</b>  This method sets the length of this stream to <var>length</var>.  Declaring the stream's length in advance can help some implementations perform more efficiently and should be done whenever possible.  It is illegal to call <code>setLength</code> with a <var>length</var> value that is less than the current flush pointer.  If the sequence is closed before <var>length</var> bytes are written, the implementation may choose unpredictable values for the remaining bytes.</p>

	@param length the new length of this stream
	**********************************************************************/

	void setLength (long length) throws IOException;



	/**********************************************************************
	Returns stream length.

	<p><b>Details:</b>  This method returns the length of this stream if it is known, or -1 otherwise.</p>

	@return the length
	**********************************************************************/

	long getLength () throws IOException;



	/**********************************************************************
	Returns write offset.

	<p><b>Details:</b>  This method returns the offset of the next BYTE that will be written if a <code>write</code> method is called.</p>

	@return the stream pointer
	**********************************************************************/

	long getPosition () throws IOException;



	/**********************************************************************
	Sets write offset.

	<p><b>Details:</b>  This method sets the write offset, or the offset of the next BYTE to be written.  If the new offset is less than the flush pointer, an error may occur.</p>

	@param newpos the new position
	**********************************************************************/

	void setPosition (long newpos) throws IOException;



	/**********************************************************************
	Writes BYTE.

	<p><b>Details:</b>  This method writes a single BYTE (<var>b</var>) to the output stream.  Only the least significant 8 bits of <var>b</var> are used.</p>

	@param b the byte
	**********************************************************************/

	void write (int b) throws IOException;



	/**********************************************************************
	Writes BYTEs.

	<p><b>Details:</b>  This method writes a sequence of BYTEs to the output stream.  The BYTEs are read from the source array (<var>src</var>) beginning at the given offset (<var>offset</var>).  <var>amount</var> BYTEs are written.  The write offset is advanced by the number of bytes written.</p>

	@param src the sequence of bytes
	@param offset offset into src
	@param amount number of bytes to write
	**********************************************************************/

	void write (byte[] src, int offset, int amount) throws IOException;



	/**********************************************************************
	Commits BYTEs.

	<p><b>Details:</b>  This method freezes and commits the bytes written up to the given offset (<var>upto</var>).  The values of bytes written to this point should no longer be modified.  Bytes whose offsets are less than the new commit offset which were never written may be filled with zeros or unpredictable values.</p>

	<p>It is acceptable to call <code>commit</code> with an <var>upto</var> value that is less than the current commit offset, but doing so has no effect.</p>

	@param upto the new commit offset
	**********************************************************************/

	void commit (long upto) throws IOException;



	/**********************************************************************
	Closes stream.

	<p><b>Details:</b>  This method commits all BYTEs that are currently buffered and releases resources associated with this implementation.</p>
	**********************************************************************/

	void close () throws IOException;



}



