///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  Notice to licensees:                                                     //
//                                                                           //
//  This source code is the exclusive, proprietary intellectual property of  //
//  Sharkysoft (sharkysoft.com).  You may view this source code as a         //
//  supplement to other product documentation, but you may not distribute    //
//  it or use it for any other purpose without written consent from          //
//  Sharkysoft.                                                              //
//                                                                           //
//  You are permitted to modify and recompile this source code, but you may  //
//  not remove this notice.  If you add features to or fix errors in this    //
//  code, please consider sharing your changes with Sharkysoft for possible  //
//  incorporation into future releases of the product.  Thanks!              //
//                                                                           //
//  For more information about Sharkysoft products and services, please      //
//  visit Sharkysoft on the web at                                           //
//                                                                           //
//       http://sharkysoft.com/                                              //
//                                                                           //
//  Thank you for using Lava!                                                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



package lava.riff;



import java.io.EOFException;
import java.io.IOException;
import lava.util.BitTwiddler;



/******************************************************************************
No summary available.

<p><b>Details:</b>  An InnerRiffChunkWriter writes data to an inner chunk in a RIFF output stream by means of its containing RiffChunkWriter, which may be an OuterRiffChunkWriter or another InnerRiffChunkWriter.  See the superclass for more details.</p>

@version 1999.09.24
@since 1999.09.24
******************************************************************************/

class InnerRiffChunkWriter
	extends RiffChunkWriter
{



	/**********************************************************************
	Parent chunk writer.

	<p><b>Details:</b>  This member points to this chunk writer's parent chunk writer.</p>
	**********************************************************************/

	final RiffChunkWriter parent;



	/**********************************************************************
	Offset in parent.

	<p><b>Details:</b>  <var>tag_ptr</var> is the offset into the parent chunk writer of this chunk's tag, where 0 is the offset of the first data byte in the parent's chunk.</p>
	**********************************************************************/

	final long tag_ptr;



	/**********************************************************************
	Current write position.

	<p><b>Details:</b>  <var>position</var> is the offset into this chunk where the next byte will be written if a write method is called, where 0 is the offset of the first data byte in this chunk.</p>
	**********************************************************************/

	long position = 0;



	/**********************************************************************
	Chunk length.

	<p><b>Details:</b>  length is this chunk's length in bytes, not including the tag, length field, and padding byte.</p>
	**********************************************************************/

	long length = 0;



	long fixed_length = 0;



	InnerRiffChunkWriter (RiffChunkWriter parent, int tag, long length) throws IOException
	{
		this . parent = parent;
		fixed_length = length;
		tag_ptr = parent . getPosition ();
		parent . writeTag (tag);
		parent . writeDword ((int) fixed_length);
	}



	long getPosition ()
	{
		return position;
	}



	void setPosition (long p) throws EOFException
	{
		if (p < 0 || fixed_length != 0 && p >= fixed_length)
			throw new EOFException ("p > fixed_length");
		position = p;
	}



	void beginwrite (long amt) throws IOException
	{
		if (fixed_length != 0 && position + amt > fixed_length)
			throw new EOFException ("attempt to write past end of chunk");
		parent . setPosition (tag_ptr + 8 + position);
	}



	void endwrite (long amt)
	{
		position += amt;
		if (position > length)
			length = position;
	}



	void writeTag (int tag) throws IOException
	{
		writeDword (BitTwiddler.reverseEndian (tag));
	}



	void writeByte (byte b) throws IOException
	{
		beginwrite (1);
		parent . writeByte (b);
		endwrite (1);
	}



	void writeWord (short word) throws IOException
	{
		beginwrite (2);
		parent . writeWord (word);
		endwrite (2);
	}



	void writeDword (int dword) throws IOException
	{
		beginwrite (4);
		parent . writeDword (dword);
		endwrite (4);
	}



	void writeBytes (byte[] src, int offset, int amount) throws IOException
	{
		beginwrite (amount);
		parent . writeBytes (src, offset, amount);
		endwrite (amount);
	}



	void writeWords (short[] src, int offset, int amount) throws IOException
	{
		beginwrite (amount * 2);
		parent . writeWords (src, offset, amount);
		endwrite (amount * 2);
	}



	void writeDwords (int[] src, int offset, int amount) throws IOException
	{
		beginwrite (amount * 4);
		parent . writeDwords (src, offset, amount);
		endwrite (amount * 4);
	}



	void flush () throws IOException
	{
		if (fixed_length > 0)
			parent . flush ();
	}



	RiffChunkWriter close () throws IOException
	{
		if (fixed_length == 0)
		{
			parent . setPosition (tag_ptr + 4);
			parent . writeDword ((int) length);
		}
		else
		{
			long missing = fixed_length - length;
			if (missing > 0)
			{
				position = length;
				beginwrite (missing);
				for (int i = 0; i < missing; ++ i)
					writeByte ((byte) 0);
				endwrite (missing);
			}
		}
		if (length % 2 == 1)
		{
			parent . setPosition (tag_ptr + 8 + length);
			parent . writeByte ((byte) 0);
		}
		return parent;
	}



}



