///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  Notice to licensees:                                                     //
//                                                                           //
//  This source code is the exclusive, proprietary intellectual property of  //
//  Sharkysoft (sharkysoft.com).  You may view this source code as a         //
//  supplement to other product documentation, but you may not distribute    //
//  it or use it for any other purpose without written consent from          //
//  Sharkysoft.                                                              //
//                                                                           //
//  You are permitted to modify and recompile this source code, but you may  //
//  not remove this notice.  If you add features to or fix errors in this    //
//  code, please consider sharing your changes with Sharkysoft for possible  //
//  incorporation into future releases of the product.  Thanks!              //
//                                                                           //
//  For more information about Sharkysoft products and services, please      //
//  visit Sharkysoft on the web at                                           //
//                                                                           //
//       http://sharkysoft.com/                                              //
//                                                                           //
//  Thank you for using Lava!                                                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



package lava.riff;



import java.io.EOFException;
import java.io.IOException;
import lava.array.LittleEndianArray;
import lava.riff.IRiffOutput;



class OuterRiffChunkWriter
	extends RiffChunkWriter
{



	/**********************************************************************
	Base output stream.

	<p><b>Details:</b>  raos is the output stream to which the RIFF data is being written.</p>
	**********************************************************************/

	private final IRiffOutput raos;



	long fixed_length = 0;



	/**********************************************************************
	Offset into output stream.

	<p><b>Details:</b>  <var>tag_ptr</var> is the offset into the base output stream of the RIFF tag.  For standard RIFF files, this value is zero, but in strange cases it may be a positive value.</p>
	**********************************************************************/

	final long tag_ptr;



	long position = 0;



	long length = 0;



	OuterRiffChunkWriter (IRiffOutput raos, long fixed_length) throws IOException
	{
		this . raos = raos;
		this . fixed_length = fixed_length;
		tag_ptr = raos . getPosition ();
		writeIntBe (Riff.RIFF_TAG);
		writeIntLe ((int) fixed_length);
		position = 0;
		length = 0;
	}



	private void writeIntBe (int i) throws IOException
	{
		raos . write (i >> 24);
		raos . write (i >> 16);
		raos . write (i >>  8);
		raos . write (i      );
	}



	private void writeIntLe (int i) throws IOException
	{
		raos . write (i      );
		raos . write (i >>  8);
		raos . write (i >> 16);
		raos . write (i >> 24);
	}



	long getPosition ()
	{
		return position;
	}



	void setPosition (long p) throws EOFException
	{
		if (p < 0 || fixed_length != 0 && p >= length)
			throw new EOFException ("p > fixed length");
		position = p;
	}



	void beginwrite (long amt) throws IOException
	{
		if (fixed_length != 0 && position + amt > fixed_length)
			throw new EOFException ("attempt to write past end of chunk");
		raos . setPosition (tag_ptr + 8 + position);
	}



	void endwrite (long amt)
	{
		position += amt;
		if (position > length)
			length = position;
	}



	void writeTag (int tag) throws IOException
	{
		beginwrite (4);
		writeIntBe (tag);
		endwrite (4);
	}



	void writeByte (byte b) throws IOException
	{
		beginwrite (1);
		raos . write (b);
		endwrite (1);
	}



	void writeWord (short word) throws IOException
	{
		beginwrite (2);
		raos . write (word);
		raos . write (word >> 8);
		endwrite (2);
	}



	void writeDword (int dword) throws IOException
	{
		beginwrite (4);
		writeIntLe (dword);
		endwrite (4);
	}



	void writeBytes (byte[] src, int offset, int amount) throws IOException
	{
		beginwrite (amount);
		raos . write (src, offset, amount);
		endwrite (amount);
	}



	void writeWords (short[] src, int offset, int amount) throws IOException
	{
		byte[] bytesrc = LittleEndianArray.split (src, offset, null, 0, amount);
		writeBytes (bytesrc, 0, bytesrc . length);
	}



	void writeDwords (int[] src, int offset, int amount) throws IOException
	{
		byte[] bytesrc = LittleEndianArray.split (src, offset, (byte[]) null, 0, amount);
		writeBytes (bytesrc, 0, bytesrc . length);
	}



	void flush () throws IOException
	{
		if (fixed_length > 0)
			raos . commit (tag_ptr + 8 + length);
	}



	RiffChunkWriter close () throws IOException
	{
//System.out.println("OuterRiffChunkWriter:close");
		if (fixed_length == 0)
		{
			raos . setPosition (tag_ptr + 4);
			writeIntLe ((int) length);
		}
		else
		{
			long missing = fixed_length - length;
			if (missing > 0)
			{
				position = length;
				beginwrite (missing);
				for (int i = 0; i < missing; ++ i)
					writeByte ((byte) 0);
				endwrite (missing);
			}
		}
		if (length % 2 == 1)
		{
			raos . setPosition (tag_ptr + 8 + length);
			raos . write ((byte) 0);
		}
		raos . close ();
		return null;
	}



}



