///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  Notice to licensees:                                                     //
//                                                                           //
//  This source code is the exclusive, proprietary intellectual property of  //
//  Sharkysoft (sharkysoft.com).  You may view this source code as a         //
//  supplement to other product documentation, but you may not distribute    //
//  it or use it for any other purpose without written consent from          //
//  Sharkysoft.                                                              //
//                                                                           //
//  You are permitted to modify and recompile this source code, but you may  //
//  not remove this notice.  If you add features to or fix errors in this    //
//  code, please consider sharing your changes with Sharkysoft for possible  //
//  incorporation into future releases of the product.  Thanks!              //
//                                                                           //
//  For more information about Sharkysoft products and services, please      //
//  visit Sharkysoft on the web at                                           //
//                                                                           //
//       http://sharkysoft.com/                                              //
//                                                                           //
//  Thank you for using Lava!                                                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



package lava.riff;



import java.io.EOFException;
import java.io.File;
import java.io.IOException;
import lava.konst.LONG;
import lava.konst.CHAR;
import lava.array.LittleEndianArray;
import lava.io.FileFormatException;
import lava.util.BitTwiddler;



/******************************************************************************
Reads data from RIFF chunk.

<p><b>Details:</b>  A <code>RiffChunkReader</code> reads and returns data from a single chunk in a RIFF file.  Since a RIFF file can have many chunks, there may be multiple chunk readers attached to the same open RIFF file.  However, because each chunk reader maintains its own file pointer, all chunk readers associated with a given RIFF stream can operate independently, and it is usually safe to process data from multiple instances at the same time (in the same thread).  Each <code>RiffChunkReader</code> maintains information about the chunk's tag, size, and location.</p>

<p>Several methods are available for reading various data types within a chunk.  Most of these methods simply return the unit of data that was requested.  However, when a <code>RiffChunkReader</code> is called on to read one of its own inner chunks, it simply advances its read pointer past the entire inner chunk, while returning a new instance of <code>RiffChunkReader</code> bound to the inner chunk.  This makes it possible to set aside the inner chunk for later processing while continuing on with the outer chunk.  Or, inner chunks can be processed immediately, in a depth-first recursive fashion.  It is up to the programmer.</p>
******************************************************************************/

public class RiffChunkReader
{



	/**********************************************************************
	The parent chunk.  If null, then this is the main RIFF chunk.
	**********************************************************************/

	private final RiffChunkReader parent;



	/**********************************************************************
	The RIFF being read.
	**********************************************************************/

	final IRiffInput rais;



	/**********************************************************************
	Chunk type identifier.

	<p><b>Details:</b>  This member is the tag associated with this chunk.  The most significant byte is the first character of the tag.</p>
	**********************************************************************/

	public final int tag;



	/**********************************************************************
	The offset into the file where the data portion of this chunk begins.
	**********************************************************************/

	private final long offset;



	/**********************************************************************
	Chunk data length.

	<p><b>Details:</b>  This member is the length of this chunk's data segment.</p>
	**********************************************************************/

	public final long size;



	/**********************************************************************
	The current reading location in the chunk, given as an offset into the file, not necessarily the chunk itself.  Each call to a read method must set the file pointer to this location before performing the read, since other readers attached to the same file might be reading from other locations.
	**********************************************************************/

	private long pointer;



	/**********************************************************************
	Opens a RIFF file and associates a new RiffChunkReader with the file's main outer chunk.  A test is performed to verify that this file contains a valid outer RIFF chunk.

	@param the RIFF file to open
	@exception IOException if an I/O error occurs
	@exception FileFormatException if the validity test fails
	**********************************************************************/

	RiffChunkReader (IRiffInput rais) throws IOException
	{
		// A RIFF file is really just a large chunk.  Here, we're reading the outermost chunk, whose tag is 'RIFF'.
		// There is no parent chunk.
		parent = null;
		// Open the (alleged) RIFF file.
		this.rais = rais;
		// Confirm that it begins with the RIFF tag.
		tag = readIntBe ();
		if (tag != Riff.RIFF_TAG)
			throw new FileFormatException ("RIFF tag missing");
		// Compare the file's size to the size reported in the RIFF header.
		size = readIntLe () & LONG.MAX_UINT;
		long estimated_file_length = 8 + size + (size & 1L);
		long actual_file_length = rais . getLength ();
		if (actual_file_length >= 0 && estimated_file_length != actual_file_length)
			throw new FileFormatException ("incorrect length field");
		// All OK!  Prepare to read the chunk contents.
		offset = 8;
		pointer = 8;
		rais . free (0, 8);
	}



	/**********************************************************************
	Initializes a new <code>RiffChunkReader</code> to read from an inner chunk.

	@param parent the reader of the outer chunk, which created this inner chunk reader
	@param offset the start of this inner chunk's data
	@param tag the tag of this inner chunk
	@param size the length of the data field of this inner chunk
	**********************************************************************/

	private RiffChunkReader (RiffChunkReader parent, long offset, int tag, long size)
	{
		this . parent = parent;
		this . rais = parent . rais;
		this . offset = offset;
		this . tag = tag;
		this . size = size;
		pointer = offset;
	}



	/**********************************************************************
	Begins reading inner chunk.

	<p><b>Details:</b>  <code>readChunk</code> does the following:</p>

	<ol>
		<li>Verfies that the data at the current read location has the characteristics of a valid inner chunk, given the outer chunks' context.
		<li>Sets the current read location to just beyond the inner chunk's end, as if all the inner chunk data had been skipped.
		<li>Returns a new <code>RiffChunkReader</code> capable of reading the inner chunk that was skipped.
	</ol>

	<p>Other than the inner chunk's header, no data from the inner chunk is actually processed</p>

	<p>If this method is called at the wrong time, i.e., when the outer chunk's pointer is not actually pointing to the start of an inner chunk, strange results can occur.  This is because there is no way to determine whether what follows is actually an inner chunk or not.  The first four bytes read are interpreted as the inner chunk's tag, and the following four bytes are read to determine the inner chunk's data length (so that it can be skipped in the outer chunk).  But if the inner chunk's length field indicates an inner chunk that is small enough to be properly nested within the outer chunk, no error can be detected.  Otherwise, an EOFException will be thrown.</p>

	@return a new chunk reader for the inner chunk
	@exception IOException if an I/O error occurs
	@exception EOFException if there is no inner chunk, or the inner chunk read is too large to be properly nested in the outer chunk
	**********************************************************************/

	public RiffChunkReader readChunk () throws IOException
	{
		// Save the current pointer so that we can reset it if the readChunk call fails.
		long saved_pointer = pointer;
		try
		{
			// Read the chunk tag:
			int tag = readTag ();
			// Read the chunk size.
			long size = readDword () & LONG.MAX_UINT;
			// Determine how many bytes must be skipped (past the byte immediately after the size field) in order to arrive at the first byte following the whole chunk:
			long skips = size + (size & 1);
			// Skip the number of bytes determined above, but save our current location, since that's where the content of this chunk begins.
			long offset = pointer;
			skipBytes (skips);
			// We succeeded in skipping past the whole chunk without errors.  That means the chunk was good.  We can now create it.
			RiffChunkReader new_chunk = new RiffChunkReader (this, offset, tag, size);
//new_chunk . debug_report ();
			return new_chunk;
		}
		catch (EOFException e)
		{
			// If we're in here, then something went wrong while we were reading the chunk.  Let's reset this chunk reader to make it the way things were before this method was called, then continue throwing the exception.
			pointer = saved_pointer;
			throw e;
		}
	}



	/**********************************************************************
	Reads chunk type identifier.

	<p><b>Details:</b>  <code>readTag</code> reads a four character tag (chunk type identifier) from the current position in the chunk.  The most significant byte of the returned value is the first character of the tag.</p>

	@return the chunk type identifier read
	@exception EOFException if reading four bytes for the tag will extend the chunk reader beyond the chunk's end
	**********************************************************************/

	public final int readTag () throws IOException
	{
		set (4);
		int tag = readIntBe ();
		pointer += 4;
		return tag;
	}



	/**********************************************************************
	Reads BYTE.

	<p><b>Details:</b>  <code>readByte</code> reads a single BYTE from the current position in the chunk.</p>

	@return the BYTE
	@exception EOFException if reading a BYTE extends the chunk reader beyond the chunk's end
	**********************************************************************/

	public final byte readByte () throws IOException
	{
		set (1);
		int b = rais . read ();
		if (b < 0)
			throw new EOFException ();
		pointer += 1;
		return (byte) b;
	}



	/**********************************************************************
	Reads WORD.

	<p><b>Details:</b>  <code>readWord</code> reads a WORD from the current position in the chunk.</p>

	@return the WORD
	@exception EOFException if reading a WORD extends the chunk reader beyond the chunk's end
	**********************************************************************/

	public final short readWord () throws IOException
	{
		set (2);
		short word = readShortLe ();
		pointer += 2;
		return word;
	}



	/**********************************************************************
	Reads DWORD.

	<p><b>Details:</b>  <code>readDword</code> reads a DWORD from the current position in the chunk.</p>

	@return the DWORD
	@exception EOFException if reading a DWORD extends the chunk reader beyond the chunk's end
	**********************************************************************/

	public final int readDword () throws IOException
	{
		set (4);
		int dword = readIntLe ();
		pointer += 4;
		return dword;
	}



	/**********************************************************************
	Reads BYTEs.

	<p><b>Details:</b>  <code>readBytes</code> reads a sequence of BYTEs and returns them in the given <code>byte</code> array (<var>bytes</var>), starting with <code>bytes [offset]</code>.  <var>amount</var> BYTEs are read.  If <var>bytes</var> is <code>null</code>, a new <code>byte</code> array is created whose length is just large enough to accomodate the request.  In either case, the array into which the BYTEs were written is returned.</p>

	@param bytes the array to copy bytes into
	@param offset offset into array where copying begins
	@param amount the number of BYTEs to read
	@return the array of BYTEs that was read
	@exception IOException if an I/O error occurs
	@exception EOFException if reading the BYTEs extends the chunk reader beyond the chunk's end
	**********************************************************************/

	public final byte[] readBytes (byte[] bytes, int offset, int amount) throws IOException
	{
		if (amount < 0)
			throw new IllegalArgumentException ("amount=" + amount);
		set (amount);
		if (bytes == null)
			bytes = new byte [offset + amount];
		readFully (bytes, offset, amount);
		return bytes;
	}



	/**********************************************************************
	Reads WORDs.

	<p><b>Details:</b>  <code>readWords</code> reads a sequence of WORDs and returns them in the given <code>short</code> array (<var>words</var>), starting with <code>words [offset]</code>.  <var>amount</var> WORDs are read.  If <var>words</var> is <code>null</code>, a new <code>short</code> array is created whose length is just large enough to accomodate the request.  In either case, the array into which the WORDs were written is returned.</p>

	@param words the array to copy words into
	@param offset offset into array where copying begins
	@param amount the number of WORDs to read
	@return the array of WORDs that was read
	@exception IOException if an I/O error occurs
	@exception EOFException if reading the WORDs extends the chunk reader beyond the chunk's end
	**********************************************************************/

	public final short[] readWords (short[] words, int offset, int amount) throws IOException
	{
		if (amount < 0)
			throw new IllegalArgumentException ("amount=" + amount);
		set (amount * 2);
		if (words == null)
			words = new short [offset + amount];
		readFully (words, offset, amount);
		return words;
	}



	/**********************************************************************
	Reads DWORDs.

	<p><b>Details:</b>  <code>readDwords</code> reads a sequence of DWORDs and returns them in the given <code>int</code> array (<var>dwords</var>), starting with <code>dwords [offset]</code>.  <var>amount</var> DWORDs are read.  If <var>dwords</var> is <code>null</code>, a new <code>int</code> array is created whose length is just large enough to accomodate the request.  In either case, the array into which the DWORDs were written is returned.</p>

	@param dwords the array to copy dwords into
	@param offset offset into array where copying begins
	@param amount the number of DWORDs to read
	@return the array of DWORDs that was read
	@exception IOException if an I/O error occurs
	@exception EOFException if reading the DWORDs extends the chunk reader beyond the chunk's end
	**********************************************************************/

	public final int[] readDwords (int[] dwords, int offset, int amount) throws IOException
	{
		if (amount < 0)
			throw new IllegalArgumentException ("amount=" + amount);
		set (amount * 4);
		if (dwords == null)
			dwords = new int [offset + amount];
		readFully (dwords, offset, amount);
		return dwords;
	}



	/**********************************************************************
	Reads string.

	<p><b>Details:</b>  <code>readString</code> reads a zero-terminated character string and returns it.</p>

	@return the string that was read
	@exception EOFException if the chunk's end is reached before the end of the string is reached
	**********************************************************************/

	public final String readString () throws IOException
	{
		long saved_pointer = pointer;
		try
		{
			StringBuffer buff = new StringBuffer ();
			while (true)
			{
				byte b = readByte ();
				if (b == 0)
					return buff . toString ();
				buff . append ((char) (b & CHAR.MAX_UBYTE));
			}
		}
		catch (IOException e)
		{
			pointer = saved_pointer;
			throw e;
		}
	}



	/**********************************************************************
	Resets read position.

	<p><b>Details:</b>  <code>reset</code> sets this chunk's read position to the beginning of the chunk's data segment.</p>
	**********************************************************************/

	public final void reset ()
	{
		pointer = offset;
	}



	/**********************************************************************
	Sets read position.

	<p><b>Details:</b>  <code>seek</code> sets this chunk's read position to the given offset relative to the beginning of the chunk's data segment.  (0 is the offset of the first data byte immediately following the tag and size DWORDs.)</p>

	@param i the new chunk pointer value
	@exception EOFException if the new chunk pointer value is beyond the end of the chunk
	**********************************************************************/

	public void seek (long i) throws IOException
	{
		if (i < 0)
			throw new IllegalArgumentException ("i=" + i);
		if (i > size)
			throw new EOFException ("seeked past end of chunk");
		pointer = offset + i;
	}



	/**********************************************************************
	Skips bytes.

	<p><b>Details:</b>  <code>skipBytes</code> skips past bytes in this chunk by advancing the chunk pointer.</p>

	@param count the number of bytes to skip
	@exception EOFException if skipping the indicated number of bytes will place the chunk pointer beyond the end of the chunk
	**********************************************************************/

	public final void skipBytes (long count) throws IOException
	{
		if (count < 0)
			throw new IllegalArgumentException ("count==" + count);
		if (! available (count))
			throw new EOFException ("passed the end of the chunk");
		pointer += count;
	}



	/**********************************************************************
	Determines if more data is available.

	<p><b>Details:</b>  <code>hasMoreData</code> determines if at least one more byte can be read from this chunk.</p>

	@return <code>true</code> if one more byte can be read, <code>false</code> otherwise
	**********************************************************************/

	public boolean hasMoreData ()
	{
		return pointer < offset + size;
	}



	/**********************************************************************
	Prepares this chunk reader to read data from its current reading position by calling the underlying source's <code>setPosition</code> method.  This method is necessary because other independent chunk readers may be attached to the same RandomAccessFile instance, and may have changed the file pointer.  The <code>read_amount</code> parameter is used to indicate the number of bytes that will be read.  If <code>read_amount</code> bytes of data are not available at the chunk's current reading position, an <code>EOFException</code> will be thrown.

	@param read_amount the number of bytes that will be read
	@exception EOFException if there is less than read_amount bytes between the chunk reader's current position and the end of the chunk
	@exception IOException if an I/O error occurs
	**********************************************************************/

	private void set (long read_amount) throws IOException
	{
		if (! available (read_amount))
			throw new EOFException ();
		rais . setPosition (pointer);
	}



	/**********************************************************************
	Determines if <code>i</code> bytes can be read from this chunk, starting at the chunk's current reading position.

	@param i the number of bytes
	@return <code>true</code> if i bytes can be read, <code>false</code> otherwise
	**********************************************************************/

	private boolean available (long i)
	{
		return pointer + i <= offset + size;
	}



//	/**********************************************************************
//	Outputs debugging information about this chunk to <code>System.out</code>.  This method should not normally be used in functional programs, but is made available for use in programs that are being debugged.
//	**********************************************************************/
//
//	public void debug_report ()
//	{
//		Stdio . printf
//		(
//			"RiffChunkReader\n" +
//			"{\n" +
//			"\ttag     = '%s'\n" +
//			"\toffset  = %lx\n" +
//			"\tsize    = %lx\n" +
//			"\tpointer = %lx\n" +
//			"}\n",
//			new Object []
//			{
//				Riff.decodeTag (tag),
//				new Long (offset),
//				new Long (size),
//				new Long (pointer)
//			}
//		);
//	}



	int readIntBe () throws IOException
	{
		byte[] buff = new byte [4];
		readFully (buff);
		return BitTwiddler.toIntBe (buff, 0);
	}



	short readShortLe () throws IOException
	{
		byte[] buff = new byte [2];
		readFully (buff);
		return BitTwiddler.toShortLe (buff, 0);
	}



	int readIntLe () throws IOException
	{
		byte[] buff = new byte [4];
		readFully (buff);
		return BitTwiddler.toIntLe (buff, 0);
	}



//	long readLongLe () throws IOException
//	{
//		byte[] buff = new byte [8];
//		readFully (buff);
//		return BitTwiddler.toLongLe (buff, 0);
//	}



	private void readFully (byte[] bytes, int offset, int amount) throws IOException
	{
		if (rais . read (bytes, offset, amount) != amount)
			throw new EOFException ();
	}



	private void readFully (short[] words, int offset, int amount) throws IOException
	{
		if (rais . read (words, offset, amount) != amount)
			throw new EOFException ();
	}



	private void readFully (int[] dwords, int offset, int amount) throws IOException
	{
		if (rais . read (dwords, offset, amount) != amount)
			throw new EOFException ();
	}



	private void readFully (byte[] buff) throws IOException
	{
		if (rais . read (buff, 0, buff . length) != buff . length)
			throw new EOFException ();
	}



	/**********************************************************************
	Unbuffers data segment.

	<p><b>Details:</b>  Clients can call <code>free</code> to indicate that they are no longer interested in the named portion of the RIFF source.  This call translates into an equivalent call to the underlying <code>IRiffInput</code>'s <code>free</code> method.  <var>from</var> indicates the offset in <em>this</em> chunk of the segment and <var>amount</var> indicates the total number of BYTEs to release.</p>

	@param from start of segment
	@param amount length of segment
	@exception IOException if an I/O error occurs
	**********************************************************************/

	public void free (long from, int amount) throws IOException
	{
		rais . free (offset + from, amount);
	}



	/**********************************************************************
	Unbuffers header.

	<p><b>Details:</b>  <code>freeHeader</code> serves the same purpose as {@link #free(long, int) <code>free</code>} but has an implied range: the 8-BYTE header for this chunk.</p>

	@exception IOException if an I/O error occurs
	**********************************************************************/

	public void freeHeader () throws IOException
	{
		rais . free (offset - 8, 8);
	}



	/**********************************************************************
	Unbuffers entire chunk.

	<p><b>Details:</b>  <code>freeAll</code> serves the same purpose as {@link #free(long, int) <code>free</code>} but has an implied range: the entire chunk, including the header and the entire data segment.</p>

	@exception IOException if an I/O error occurs
	**********************************************************************/

	public void freeAll () throws IOException
	{
		rais . free (offset - 8, size + 8);
	}



}



