///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  Notice to licensees:                                                     //
//                                                                           //
//  This source code is the exclusive, proprietary intellectual property of  //
//  Sharkysoft (sharkysoft.com).  You may view this source code as a         //
//  supplement to other product documentation, but you may not distribute    //
//  it or use it for any other purpose without written consent from          //
//  Sharkysoft.                                                              //
//                                                                           //
//  You are permitted to modify and recompile this source code, but you may  //
//  not remove this notice.  If you add features to or fix errors in this    //
//  code, please consider sharing your changes with Sharkysoft for possible  //
//  incorporation into future releases of the product.  Thanks!              //
//                                                                           //
//  For more information about Sharkysoft products and services, please      //
//  visit Sharkysoft on the web at                                           //
//                                                                           //
//       http://sharkysoft.com/                                              //
//                                                                           //
//  Thank you for using Lava!                                                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



package lava.riff;



import lava.array.LittleEndianArray;
import java.io.IOException;
import java.io.InputStream;
import lava.konst.INT;
import lava.konst.LONG;
import lava.array.ByteArray32;



/******************************************************************************
InputStream based RIFF data source.

<p><b>Details:</b>  RiffInputStream is a <code>java.io.InputStream</code>-<wbr>based implementation of <code>IRiffInput</code>.  In order to enable seeking while reading from the stream, <code>RiffInputStream</code> buffers every byte it reads from the underlying stream.  Because this can require large amounts of memory for large streams, the client should call the <code>free</code> method whenever possible in order to release portions of the buffer that are no longer needed.</p>

@since 1999.10.28
@author Sharky
******************************************************************************/

public class RiffInputStream
	implements IRiffInput
{



	private final InputStream is;



	private long position = 0;



	private long advance = 0;



	private long length = LONG.MAX_SLONG;



	private final ByteArray32 memory = new ByteArray32 ((byte) 0);



	/**********************************************************************
	Sets base InputStream.

	<p><b>Details:</b>  This constructor sets the base InputStream to <var>is</var>.</p>

	@param is the base InputStream
	**********************************************************************/

	public RiffInputStream (InputStream is)
	{
		this . is = is;
	}



	public long getLength () throws IOException
	{
		return length;
	}



	public long getPosition () throws IOException
	{
		return position;
	}



	public void setPosition (long newpos) throws IOException
	{
		position = newpos;
	}



	private void bufferAdvance (int amount) throws IOException
	{
		long goal = position + amount;
		if (goal < advance)
			return;
		byte[] buff = new byte [1024];
		while (advance < goal)
		{
			amount = is . read (buff, 0, buff . length);
			if (amount < 0)
			{
				length = advance;
				break;
			}
			memory . set ((int) advance, buff, 0, amount);
			advance += amount;
		}
	}



	public int read () throws IOException
	{
		bufferAdvance (1);
		if (position >= length)
			return -1;
		byte b = memory . get ((int) position);
		++ position;
		return b & INT.MAX_UBYTE;
	}



	public int read (byte[] dest, int offset, int amount) throws IOException
	{
		bufferAdvance (amount);
		if (position >= length)
			return -1;
		if (position + amount > length)
			amount = (int) (length - position);
		memory . get ((int) position, dest, offset, amount);
		position += amount;
		return amount;
	}



	public int read (short[] dest, int offset, int amount) throws IOException
	{
		int amt2 = amount * 2;
		byte[] bytes = new byte [amt2];
		int r = read (bytes, 0, amt2);
		if (r < 0)
			return r;
		r &= ~ 1;
		LittleEndianArray.concat (bytes, 0, dest, offset, r);
		return r / 2;
	}



	public int read (int[] dest, int offset, int amount) throws IOException
	{
		int amt4 = amount * 4;
		byte[] bytes = new byte [amt4];
		int r = read (bytes, 0, amt4);
		if (r < 0)
			return r;
		r &= ~ 3;
		LittleEndianArray.concat (bytes, 0, dest, offset, r);
		return r / 4;
	}



	public void free (long offset, long amount)
	{
//System.out.println("free("+offset+","+amount+")");
		memory . fill ((int) offset, (int) amount, (byte) 0);
		memory . compact ();
	}



	public void close () throws IOException
	{
		is . close ();
	}



}



