///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  Notice to licensees:                                                     //
//                                                                           //
//  This source code is the exclusive, proprietary intellectual property of  //
//  Sharkysoft (sharkysoft.com).  You may view this source code as a         //
//  supplement to other product documentation, but you may not distribute    //
//  it or use it for any other purpose without written consent from          //
//  Sharkysoft.                                                              //
//                                                                           //
//  You are permitted to modify and recompile this source code, but you may  //
//  not remove this notice.  If you add features to or fix errors in this    //
//  code, please consider sharing your changes with Sharkysoft for possible  //
//  incorporation into future releases of the product.  Thanks!              //
//                                                                           //
//  For more information about Sharkysoft products and services, please      //
//  visit Sharkysoft on the web at                                           //
//                                                                           //
//       http://sharkysoft.com/                                              //
//                                                                           //
//  Thank you for using Lava!                                                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



package lava.riff;



import java.io.IOException;
import lava.riff.IRiffOutput;



/******************************************************************************
Generates RIFF stream.

<p><b>Details:</b>  <code>RiffStreamWriter</code> generates RIFF streams (duh!).  Data is stored in a tagged, nested chunk structure, as specified by the RIFF standard.  Most of the details for creating and nesting chunks are handled automatically, and the programmer needs only to call <code>beginChunk</code>, <code>endChunk</code>, and the data writing methods in order to produce a beautifully structured RIFF stream.</p>
******************************************************************************/

public class RiffStreamWriter
{



	private RiffChunkWriter rc;



	/**********************************************************************
	Writes RIFF header.

	<p><b>Details:</b>  This constructor initializes a new <code>RiffStreamWriter</code> so that all subsequent RIFF data is written to the given <code>IRiffOutput</code> (<var>raos</var>).</p>

	<p>The length parameter can be used to declare the intended data length of the RIFF file's outer chunk, if that value is known.  In most cases, however, because <code>RiffStreamWriter</code> is meant to insulate the API user from RIFF details, the length of the RIFF stream will not be known until the entire stream has been generated.  Setting this parameter to 0 indicates that the data length of the outer chunk is unknown.</p>

	<p>Because the second DWORD in a RIFF file specifies the RIFF file's data length, not knowing the data length ahead of time prevents a <code>RiffStreamWriter</code> from committing any data to the output stream until the entire RIFF structure has been generated.  This can require significant memory resources for large streams, such as Microsoft Wave or AVI files.</p>

	@param raos the output stream
	@param length the data length
	@IOException if an I/O error occurs
	**********************************************************************/

	public RiffStreamWriter (IRiffOutput raos, long length) throws IOException
	{
		rc = new OuterRiffChunkWriter (raos, length);
		rc . flush ();
	}



	/**********************************************************************
	Creates inner chunk.

	<p><b>Details:</b>  <code>beginChunk</code> creates a new inner chunk that is nested within the currently open outer chunk.  The new inner chunk is tagged with <var>tag</var>.  <var>length</var> indicates the length of the inner chunk's data field.  If the inner chunk's data field length is unknown at the time <code>beginChunk</code> is called, the client should set <var>length</var> to 0.  Setting <var>length</var> to 0 causes this RiffStreamWriter to wait until after the inner chunk is closed before writing the inner chunk's header.</p>

	@param tag the inner chunk's tag
	@param length the predicted data length
	@IOException if an I/O error occurs
	**********************************************************************/

	public void beginChunk (int tag, long length) throws IOException
	{
		rc = new InnerRiffChunkWriter (rc, tag, length);
		rc . flush ();
	}



	/**********************************************************************
	Terminates current chunk.

	<p><b>Details:</b>  <code>endChunk</code> terminates the currently open chunk and prepares to continue writing data in the parent chunk immediately after the last byte of the closed inner chunk.  If the current chunk is the RIFF file's outer chunk, then this method wraps things up and closes the RIFF output stream entirely.  If the chunk being closed did not have a predeclared length, the length field in the inner chunk's header is updated before closing the chunk.</p>

	@IOException if an I/O error occurs
	**********************************************************************/

	public void endChunk () throws IOException
	{
		rc = rc . close ();
		if (rc != null)
			rc . flush ();
	}



	/**********************************************************************
	Closes RIFF stream.

	<p><b>Details:</b>  <code>close</code> closes all currently open, nested chunks and completes the RIFF stream, updating the headers to all chunks structures as needed.  The underlying data sink is closed as well.</p>

	@IOException if an I/O error occurs
	**********************************************************************/

	public void close () throws IOException
	{
		while (rc != null)
			endChunk ();
	}



	/**********************************************************************
	Writes tag.

	<p><b>Details:</b>  <code>writeTag</code> outputs the 4 bytes of <var>tag</var> in big endian order.  This method is typically used to output the four character tags which appear in the headers to each chunk, but it may be used for other purposes as well.</p>

	@param tag the tag

	@IOException if an I/O error occurs
	**********************************************************************/

	public void writeTag (int tag) throws IOException
	{
		rc . writeTag (tag);
		rc . flush ();
	}



	/**********************************************************************
	Writes BYTE.

	<p><b>Details:</b> This method writes a single BYTE to the current chunk.</p>

	@param b the BYTE
	@exception IOException if an I/O error occurs
	**********************************************************************/

	public void writeByte (byte b) throws IOException
	{
		rc . writeWord (b);
		rc . flush ();
	}



	/**********************************************************************
	Writes WORD.

	<p><b>Details:</b> This method writes a single WORD to the current chunk.</p>

	@param word the WORD
	@exception IOException if an I/O error occurs
	**********************************************************************/

	public void writeWord (short word) throws IOException
	{
		rc . writeWord (word);
		rc . flush ();
	}



	/**********************************************************************
	Writes DWORD.

	<p><b>Details:</b> This method writes a single DWORD to the current chunk.</p>

	@param dword the DWORD
	@exception IOException if an I/O error occurs
	**********************************************************************/

	public void writeDword (int dword) throws IOException
	{
		rc . writeDword (dword);
		rc . flush ();
	}



	/**********************************************************************
	Writes BYTEs.

	<p><b>Details:</b>  <code>writeBytes</code> writes a sequence of BYTEs to the current chunk, taking them from the given <code>byte</code> array (<var>src</var>), starting with <code>src [offset]</code>.  <var>amount</var> BYTEs are written.</p>

	@param src    BYTEs to write
	@param offset offset into array
	@param amount number of BYTEs to write
	@exception IOException  if an I/O error occurs
	@exception EOFException if reading the BYTEs extends the chunk reader beyond the chunk's end
	**********************************************************************/

	public void writeBytes (byte[] src, int offset, int amount) throws IOException
	{
		rc . writeBytes (src, offset, amount);
		rc . flush ();
	}



	/**********************************************************************
	Writes WORDs.

	<p><b>Details:</b>  <code>writeWords</code> writes a sequence of WORDs to the current chunk, taking them from the given <code>short</code> array (<var>src</var>), starting with <code>src [offset]</code>.  <var>amount</var> WORDs are written.</p>

	@param src    WORDs to write
	@param offset offset into array
	@param amount number of WORDs to write
	@exception IOException  if an I/O error occurs
	@exception EOFException if reading the WORDs extends the chunk reader beyond the chunk's end
	**********************************************************************/

	public void writeWords (short[] src, int offset, int amount) throws IOException
	{
		rc . writeWords (src, offset, amount);
		rc . flush ();
	}



	/**********************************************************************
	Writes DWORDs.

	<p><b>Details:</b>  <code>writeDwords</code> writes a sequence of DWORDs to the current chunk, taking them from the given <code>int</code> array (<var>src</var>), starting with <code>src [offset]</code>.  <var>amount</var> DWORDs are written.</p>

	@param src    DWORDs to write
	@param offset offset into array
	@param amount number of DWORDs to write
	@exception IOException  if an I/O error occurs
	@exception EOFException if reading the DWORDs extends the chunk reader beyond the chunk's end
	**********************************************************************/

	public void writeDwords (int[] src, int offset, int amount) throws IOException
	{
		rc . writeDwords (src, offset, amount);
		rc . flush ();
	}



	/**********************************************************************
	Finalizes RIFF stream.

	<p><b>Details:</b>  <code>finalize</code> calls {@link #close() <code>close</code>}.</p>
	**********************************************************************/

	protected void finalize () throws Throwable
	{
		close ();
	}



}



