///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  Notice to licensees:                                                     //
//                                                                           //
//  This source code is the exclusive, proprietary intellectual property of  //
//  Sharkysoft (sharkysoft.com).  You may view this source code as a         //
//  supplement to other product documentation, but you may not distribute    //
//  it or use it for any other purpose without written consent from          //
//  Sharkysoft.                                                              //
//                                                                           //
//  You are permitted to modify and recompile this source code, but you may  //
//  not remove this notice.  If you add features to or fix errors in this    //
//  code, please consider sharing your changes with Sharkysoft for possible  //
//  incorporation into future releases of the product.  Thanks!              //
//                                                                           //
//  For more information about Sharkysoft products and services, please      //
//  visit Sharkysoft on the web at                                           //
//                                                                           //
//       http://sharkysoft.com/                                              //
//                                                                           //
//  Thank you for using Lava!                                                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



package lava.riff.wave;



import java.io.IOException;
import lava.io.FileFormatException;
import lava.util.BitTwiddler;
import lava.riff.RiffChunkReader;
import lava.riff.RiffStreamReader;



/******************************************************************************
Format chunk for Microsoft PCM .wav files.

<p><b>Details:</b>  A <code>PcmWaveFormatChunk</code> represents the format chunk of a RIFF-Wave file where the sample data is encoded using the Microsoft PCM format.</p>
******************************************************************************/

public class PcmWaveFormatChunk
	extends WaveFormatChunk
{



	/**********************************************************************
	wFormat constant.

	<p><b>Details:</b>  <code>WAVE_FORMAT_PCM</code> is the WORD constant used in wave files to identify the Microsoft Pulse Code Modulation (PCM) format.</p>
	**********************************************************************/

	static final short WAVE_FORMAT_PCM = RIFF_WAVE.WAVE_FORMAT_PCM;



	/**********************************************************************
	Number of bits per channel sample.

	<p><b>Details:</b>  This member reflects the wBitsPerSample field in a PCM format chunk, or the number of bits per channel sample.  This is the number of bits used to sample a <em>single</em> channel, even if <code>wChannels &gt; 1</code>.  It is assumed that all channels have the same bits per channel sample.</p>
	**********************************************************************/

	public short wBitsPerSample;



	/**********************************************************************
	Initializes members from RiffChunkReader.

	<p><b>Details:</b>  This constructor initializes this instance by reading data from the given chunk reader.</p>

	@param rc the chunk reader
	@exception IOException if an I/O error occurs
	**********************************************************************/

	PcmWaveFormatChunk (RiffChunkReader rc) throws IOException
	{
		super (rc);
		if (wFormatTag != WAVE_FORMAT_PCM)
			throw new FileFormatException ("wFormat == " + wFormatTag);
		wBitsPerSample = rc . readWord ();
		validateFields ();
		if (wBitsPerSample < 1)
			throw new FileFormatException ("wBitsPerSample == " + wBitsPerSample);
	}



	/**********************************************************************
	Sets sampling parameters.

	<p><b>Details:</b>  This constructor sets the members of this format chunk to reflect the given sampling parameters, namely, the number of channels (<var>channels</var>), the bits per channel sample (<var>bpcs</var>), and the number of samples per second (<var>rate</var>).</p>

	@param channels the number of channels
	@param bits per channel sample
	@param sample rate
	**********************************************************************/

	public PcmWaveFormatChunk (int channels, int bpcs, int rate)
	{
		wFormatTag = RIFF_WAVE.WAVE_FORMAT_PCM;
		wChannels = (short) channels;
		wBitsPerSample = (short) bpcs;
		dwSamplesPerSec = rate;
		wBlockAlign = (short) getSampleSize ();
		dwAvgBytesPerSec = rate * wBlockAlign;
	}



	// javadoc comment inherited from superclass
	public final int getSampleSize ()
	{
		return (wBitsPerSample + 7) / 8 * wChannels;
	}



	/**********************************************************************
	Serializes chunk.

	<p><b>Details:</b>  <code>toBytes</code> serializes this chunk into a string of bytes, as it would appear in a .wav file.</p>

	@return this chunk in byte array form
	**********************************************************************/

	public byte[] toBytes ()
	{
		byte[] superbytes = super . toBytes ();
		byte[] thisbytes = new byte [superbytes . length + 2];
		System.arraycopy (superbytes, 0, thisbytes, 0, superbytes . length);
		BitTwiddler.toBytesLe (wBitsPerSample, thisbytes, superbytes . length);
		return thisbytes;
	}



	// javadocs inherited from superclass
	public String toString ()
	{
		return
			"PcmWaveFormatChunk" +
			"(" +
				super . toString () + "; " +
				"wBitsPerSample: " + wBitsPerSample +
			")";
	}



}



