///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  Notice to licensees:                                                     //
//                                                                           //
//  This source code is the exclusive, proprietary intellectual property of  //
//  Sharkysoft (sharkysoft.com).  You may view this source code as a         //
//  supplement to other product documentation, but you may not distribute    //
//  it or use it for any other purpose without written consent from          //
//  Sharkysoft.                                                              //
//                                                                           //
//  You are permitted to modify and recompile this source code, but you may  //
//  not remove this notice.  If you add features to or fix errors in this    //
//  code, please consider sharing your changes with Sharkysoft for possible  //
//  incorporation into future releases of the product.  Thanks!              //
//                                                                           //
//  For more information about Sharkysoft products and services, please      //
//  visit Sharkysoft on the web at                                           //
//                                                                           //
//       http://sharkysoft.com/                                              //
//                                                                           //
//  Thank you for using Lava!                                                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



package lava.riff.wave;



import lava.konst.FLOAT;
import lava.konst.DOUBLE;
import lava.array.ArrayCaster;
import java.io.IOException;
import lava.array.LittleEndianArray;
import lava.io.FileFormatException;
import lava.riff.IRiffInput;
import lava.riff.RiffChunkReader;



/******************************************************************************
Reads standard Microsoft PCM .wav file.

<p><b>Details:</b>  This class parses a standard, uncompressed .wav file whose sample data is encoded in the Microsoft Pulse Code Modulation (PCM) format.  This is the format most commonly used for .wav files.</p>

@author Sharky
******************************************************************************/

public class PcmWaveStreamReader
	extends WaveStreamReader
{



	/**********************************************************************
	Sets input source.

	<p><b>Details:</b>  This constructor sets the .wav file input source (<var>rais</var>) and parses the format chunk.</p>

	@param rais the input source
	@exception FileFormatException if the PCM .wav stream is invalid
	@exception IOException if an I/O error occurs
	**********************************************************************/

	public PcmWaveStreamReader (IRiffInput rais) throws IOException
	{
		super (rais);
		RiffChunkReader rcr = getChunkReader (findChunk (0, RIFF_WAVE.FMT_TAG));
		if (rcr == null)
			throw new FileFormatException ("rais has no FMT chunk.");
		fmt = new PcmWaveFormatChunk (rcr);
		rcr . freeAll ();
	}



	// javadoc comment inherited from superclass
	public int getBitsPerChannelSample ()
	{
		return ((PcmWaveFormatChunk) fmt) . wBitsPerSample;
	}



	// javadoc comment inherited from superclass
	public long countSamples () throws IOException
	{
		return getDataChunk () . size / fmt . getSampleSize ();
	}



	private RiffChunkReader data_chunk = null;



	private RiffChunkReader getDataChunk () throws IOException
	{
		if (data_chunk == null)
		{
			data_chunk = getChunkReader (findChunk (0, RIFF_WAVE.DATA_TAG));
			if (data_chunk == null)
				throw new FileFormatException ("WAVE file has no DATA chunk");
			data_chunk . freeHeader ();
		}
		return data_chunk;
	}



	// javadoc comment inherited from superclass
	public Object readSamples (long when, Object dest, int dest_from, int duration) throws IOException
	{
		final int element_size = fmt . getSampleSize () / fmt . wChannels;
		// pos: chunk offset
		long pos = when * fmt . wChannels * element_size;
		getDataChunk () . seek (pos);
		switch (element_size)
		{
		case 1:
			return read_channel_samples1 (dest, dest_from, duration);
		case 2:
			return read_channel_samples2 (dest, dest_from, duration);
		case 3:
			return read_channel_samples3 (dest, dest_from, duration);
		case 4:
			return read_channel_samples4 (dest, dest_from, duration);
		default:
			throw new lava.NotImplementedException ("channel sample size not supported: " + getBitsPerChannelSample ());
		}
	}



	private Object read_channel_samples1 (Object dest, int dest_from, int duration) throws IOException
	{
		byte[] bdest;
		if (dest instanceof byte[])
			bdest = (byte[]) dest;
		else
			bdest = null; //new byte [duration * fmt . wChannels * element_size];
		final int limit = duration * fmt . wChannels;
		bdest = getDataChunk () . readBytes (bdest, dest_from, limit);
		if (bdest == dest || dest == null)
			return bdest;
		switch (ArrayCaster.getPrimitiveElementType (dest))
		{
		case 2:
			WaveSampleConverter.toShorts (bdest, dest_from, (short[]) dest, dest_from, limit);
			break;
		case 4:
			WaveSampleConverter.toInts (bdest, dest_from, (int[]) dest, dest_from, limit);
			break;
		case 8:
			WaveSampleConverter.toLongs (bdest, dest_from, (long[]) dest, dest_from, limit);
			break;
		case 6:
			WaveSampleConverter.toFloats (bdest, dest_from, (float[]) dest, dest_from, limit);
			break;
		case 10:
			WaveSampleConverter.toDoubles (bdest, dest_from, (double[]) dest, dest_from, limit);
			break;
		default:
			throw new IllegalArgumentException ("dest");
		}
		return dest;
	}



	private Object read_channel_samples2 (Object dest, int dest_from, int duration) throws IOException
	{
		short[] sdest;
		if (dest instanceof short[])
			sdest = (short[]) dest;
		else
			sdest = null;
		final int limit = duration * fmt . wChannels;
		sdest = getDataChunk () . readWords (sdest, dest_from, limit);
		if (sdest == dest || dest == null)
			return sdest;
		switch (ArrayCaster.getPrimitiveElementType (dest))
		{
		case 1:
			WaveSampleConverter.toBytes (sdest, dest_from, (byte[]) dest, dest_from, limit);
			break;
		case 4:
			WaveSampleConverter.toInts (sdest, dest_from, (int[]) dest, dest_from, limit);
			break;
		case 8:
			WaveSampleConverter.toLongs (sdest, dest_from, (long[]) dest, dest_from, limit);
			break;
		case 6:
			WaveSampleConverter.toFloats (sdest, dest_from, (float[]) dest, dest_from, limit);
			break;
		case 10:
			WaveSampleConverter.toDoubles (sdest, dest_from, (double[]) dest, dest_from, limit);
			break;
		default:
			throw new IllegalArgumentException ("dest");
		}
		return dest;
	}



	private Object read_channel_samples3 (Object dest, int dest_from, int duration) throws IOException
	{
		final int limit = duration * fmt . wChannels;
		byte[] bdest = getDataChunk () . readBytes (null, 0, limit * 3);
		if (dest == null)
			return WaveSampleConverter.toInts3 (bdest, 0, null, dest_from, limit);
		switch (ArrayCaster.getPrimitiveElementType (dest))
		{
		case 1:
			WaveSampleConverter.toBytes3 (bdest, 0, (byte[]) dest, dest_from, limit);
			break;
		case 2:
			WaveSampleConverter.toShorts3 (bdest, 0, (short[]) dest, dest_from, limit);
			break;
		case 4:
			WaveSampleConverter.toInts3 (bdest, 0, (int[]) dest, dest_from, limit);
			break;
		case 8:
			WaveSampleConverter.toLongs3 (bdest, 0, (long[]) dest, dest_from, limit);
			break;
		case 6:
			WaveSampleConverter.toFloats3 (bdest, 0, (float[]) dest, dest_from, limit);
			break;
		case 10:
			WaveSampleConverter.toDoubles3 (bdest, 0, (double[]) dest, dest_from, limit);
			break;
		default:
			throw new IllegalArgumentException ("dest (" + dest + ")");
		}
		return dest;
	}



	private Object read_channel_samples4 (Object dest, int dest_from, int duration) throws IOException
	{
		int[] idest;
		if (dest instanceof int[])
			idest = (int[]) dest;
		else
			idest = null;
		final int limit = duration * fmt . wChannels;
		idest = getDataChunk () . readDwords (idest, dest_from, limit);
		if (idest == dest || dest == null)
			return idest;
		switch (ArrayCaster.getPrimitiveElementType (dest))
		{
		case 1:
			WaveSampleConverter.toBytes (idest, dest_from, (byte[]) dest, dest_from, limit);
			break;
		case 2:
			WaveSampleConverter.toShorts (idest, dest_from, (short[]) dest, dest_from, limit);
			break;
		case 8:
			WaveSampleConverter.toLongs (idest, dest_from, (long[]) dest, dest_from, limit);
			break;
		case 6:
			WaveSampleConverter.toFloats (idest, dest_from, (float[]) dest, dest_from, limit);
			break;
		case 10:
			WaveSampleConverter.toDoubles (idest, dest_from, (double[]) dest, dest_from, limit);
			break;
		default:
			throw new IllegalArgumentException ("dest");
		}
		return dest;
	}



	// javadoc inherited
	public void freeSamples (long when, int duration) throws IOException
	{
		RiffChunkReader rcr = getDataChunk ();
		final int element_size = fmt . getSampleSize () / fmt . wChannels;
		long pos = when * fmt . wChannels * element_size;
		int bytes = duration * fmt . wChannels * element_size;
		rcr . free (pos, bytes);
//System.out.println("freeSamples:("+pos+","+bytes+")");
	}



	public int getChannelSampleSize () throws IOException
	{
		int bps = ((PcmWaveFormatChunk) getFormatChunk ()) . wBitsPerSample;
		if (bps <= 8)
			return 1;
		if (bps <= 16)
			return 2;
		if (bps <= 32)
			return 4;
		throw new lava.UnreachableCodeException ();
	}



}



