///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  Notice to licensees:                                                     //
//                                                                           //
//  This source code is the exclusive, proprietary intellectual property of  //
//  Sharkysoft (sharkysoft.com).  You may view this source code as a         //
//  supplement to other product documentation, but you may not distribute    //
//  it or use it for any other purpose without written consent from          //
//  Sharkysoft.                                                              //
//                                                                           //
//  You are permitted to modify and recompile this source code, but you may  //
//  not remove this notice.  If you add features to or fix errors in this    //
//  code, please consider sharing your changes with Sharkysoft for possible  //
//  incorporation into future releases of the product.  Thanks!              //
//                                                                           //
//  For more information about Sharkysoft products and services, please      //
//  visit Sharkysoft on the web at                                           //
//                                                                           //
//       http://sharkysoft.com/                                              //
//                                                                           //
//  Thank you for using Lava!                                                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



package lava.riff.wave;



import lava.konst.FLOAT;
import lava.konst.DOUBLE;
import lava.array.ArrayCaster;
import lava.riff.IRiffOutput;
import java.io.IOException;



/******************************************************************************
Writes standard Microsoft PCM wave files.

<p><b>Details:</b>  No details available.</p>

@version 1999
@since 1999
******************************************************************************/

public class PcmWaveStreamWriter
	extends WaveStreamWriter
{



	private final PcmWaveFormatChunk fmt;



	public PcmWaveStreamWriter (IRiffOutput raos, PcmWaveFormatChunk fmt) throws IOException
	{
		super (raos, fmt);
		fmt . validateFields ();
		if (fmt . getSampleSize () / fmt . wChannels > 4)
			throw new lava.NotSupportedException ("fmt.wBitsPerSample == " + fmt . wBitsPerSample);
		this . fmt = (PcmWaveFormatChunk) fmt . clone ();
	}



	private boolean writing_samples = false;



	public synchronized void beginSamples (long duration) throws IOException
	{
		if (writing_samples)
			throw new IllegalStateException ("beginSamples has already been called.");
		rsw . beginChunk (RIFF_WAVE.DATA_TAG, fmt . getSampleSize () * duration);
		writing_samples = true;
	}



	public synchronized void writeSamples (Object samples, int samples_from, int duration) throws IOException
	{
		if (! writing_samples)
			throw new IllegalStateException ("beginSamples must be called before writeSamples.");
		final int element_size = fmt . getSampleSize () / fmt . wChannels;
		switch (element_size)
		{
		case 1:
			write_channel_samples_1 (samples, samples_from, duration);
			break;
		case 2:
			write_channel_samples_2 (samples, samples_from, duration);
			break;
		case 3:
			write_channel_samples_3 (samples, samples_from, duration);
			break;
		case 4:
			write_channel_samples_4 (samples, samples_from, duration);
			break;
		default:
			throw new lava.NotSupportedException ("channel sample size not supported: " + fmt . wBitsPerSample);
		}
	}



	private void write_channel_samples_1 (Object samples, int samples_from, int duration) throws IOException
	{
		final int length = duration * fmt . wChannels;
		byte[] bsrc;
		switch (ArrayCaster.getPrimitiveElementType (samples))
		{
		case 1:
			bsrc = (byte[]) samples;
			break;
		case 2:
			bsrc = WaveSampleConverter.toBytes ((short[]) samples, samples_from, null, samples_from, length);
			break;
		case 4:
			bsrc = WaveSampleConverter.toBytes ((int[]) samples, samples_from, null, samples_from, length);
			break;
		case 8:
			bsrc = WaveSampleConverter.toBytes ((long[]) samples, samples_from, null, samples_from, length);
			break;
		case 6:
			bsrc = WaveSampleConverter.toBytes ((float[]) samples, samples_from, null, samples_from, length);
			break;
		case 10:
			bsrc = WaveSampleConverter.toBytes ((double[]) samples, samples_from, null, samples_from, length);
			break;
		default:
			throw new IllegalArgumentException ("samples");
		}
		WaveSampleRounder.round (bsrc, samples_from, bsrc, samples_from, length, fmt . wBitsPerSample);
		rsw . writeBytes (bsrc, samples_from, length);
	}



	private void write_channel_samples_2 (Object samples, int samples_from, int duration) throws IOException
	{
		final int length = duration * fmt . wChannels;
		short[] ssrc;
		switch (ArrayCaster.getPrimitiveElementType (samples))
		{
		case 1:
			ssrc = WaveSampleConverter.toShorts ((byte[]) samples, samples_from, null, samples_from, length);
			break;
		case 2:
			ssrc = (short[]) samples;
			break;
		case 4:
			ssrc = WaveSampleConverter.toShorts ((int[]) samples, samples_from, null, samples_from, length);
			break;
		case 8:
			ssrc = WaveSampleConverter.toShorts ((long[]) samples, samples_from, null, samples_from, length);
			samples_from = 0;
			break;
		case 6:
			ssrc = WaveSampleConverter.toShorts ((float[]) samples, samples_from, null, samples_from, length);
			break;
		case 10:
			ssrc = WaveSampleConverter.toShorts ((double[]) samples, samples_from, null, samples_from, length);
			break;
		default:
			throw new IllegalArgumentException ("samples");
		}
		WaveSampleRounder.round (ssrc, samples_from, ssrc, samples_from, length, fmt . wBitsPerSample);
		rsw . writeWords (ssrc, samples_from, length);
	}



	private void write_channel_samples_3 (Object samples, int samples_from, int duration) throws IOException
	{
		final int length = duration * fmt . wChannels;
		int[] isrc;
		switch (ArrayCaster.getPrimitiveElementType (samples))
		{
		case 1:
			isrc = WaveSampleConverter.toInts ((byte[]) samples, samples_from, null, samples_from, length);
			break;
		case 2:
			isrc = WaveSampleConverter.toInts ((short[]) samples, samples_from, null, samples_from, length);
			break;
		case 4:
			isrc = (int[]) samples;
			break;
		case 8:
			isrc = WaveSampleConverter.toInts ((long[]) samples, samples_from, null, samples_from, length);
			break;
		case 6:
			isrc = WaveSampleConverter.toInts ((float[]) samples, samples_from, null, samples_from, length);
			break;
		case 10:
			isrc = WaveSampleConverter.toInts ((double[]) samples, samples_from, null, samples_from, length);
			break;
		default:
			throw new IllegalArgumentException ("samples");
		}
		WaveSampleRounder.round (isrc, samples_from, isrc, samples_from, length, fmt . wBitsPerSample);
		byte[] bsrc = WaveSampleConverter.to3Bytes (isrc, samples_from, null, 0, length);
		rsw . writeBytes (bsrc, 0, length * 3);
	}



	private void write_channel_samples_4 (Object samples, int samples_from, int duration) throws IOException
	{
		final int length = duration * fmt . wChannels;
		int[] isrc;
		switch (ArrayCaster.getPrimitiveElementType (samples))
		{
		case 1:
			isrc = WaveSampleConverter.toInts ((byte[]) samples, samples_from, null, samples_from, length);
			break;
		case 2:
			isrc = WaveSampleConverter.toInts ((short[]) samples, samples_from, null, samples_from, length);
			break;
		case 4:
			isrc = (int[]) samples;
			break;
		case 8:
			isrc = WaveSampleConverter.toInts ((long[]) samples, samples_from, null, samples_from, length);
			break;
		case 6:
			isrc = WaveSampleConverter.toInts ((float[]) samples, samples_from, null, samples_from, length);
			break;
		case 10:
			isrc = WaveSampleConverter.toInts ((double[]) samples, samples_from, null, samples_from, length);
			break;
		default:
			throw new IllegalArgumentException ("samples");
		}
		WaveSampleRounder.round (isrc, samples_from, isrc, samples_from, length, fmt . wBitsPerSample);
		rsw . writeDwords (isrc, samples_from, length);
	}



	public synchronized void endSamples () throws IOException
	{
		rsw . endChunk ();
		writing_samples = false;
	}



	public void close () throws IOException
	{
		rsw . close ();
	}



}



