///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  Notice to licensees:                                                     //
//                                                                           //
//  This source code is the exclusive, proprietary intellectual property of  //
//  Sharkysoft (sharkysoft.com).  You may view this source code as a         //
//  supplement to other product documentation, but you may not distribute    //
//  it or use it for any other purpose without written consent from          //
//  Sharkysoft.                                                              //
//                                                                           //
//  You are permitted to modify and recompile this source code, but you may  //
//  not remove this notice.  If you add features to or fix errors in this    //
//  code, please consider sharing your changes with Sharkysoft for possible  //
//  incorporation into future releases of the product.  Thanks!              //
//                                                                           //
//  For more information about Sharkysoft products and services, please      //
//  visit Sharkysoft on the web at                                           //
//                                                                           //
//       http://sharkysoft.com/                                              //
//                                                                           //
//  Thank you for using Lava!                                                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



package lava.riff.wave;



import lava.riff.RiffChunkReader;
import java.io.IOException;



/******************************************************************************
Parses associated data list chunk.

<p><b>Details:</b>  A <code>WaveAdtlChunkReader</code> parses and decodes certain elements from a wave stream's ADTL (Associated Data List) chunk.  This can be useful for decoding cue point information.</p>
******************************************************************************/

public class WaveAdtlChunkReader
{



	private final RiffChunkReader rcr;



	private WaveAdtlChunkReader (RiffChunkReader chunk)
	{
		rcr = chunk;
	}



	static WaveAdtlChunkReader create (RiffChunkReader chunk) throws IOException
	{
		if (chunk . tag != RIFF_WAVE.LIST_TAG)
			return null;
		chunk . reset ();
		if (chunk . readTag () != RIFF_WAVE.ADTL_TAG)
			return null;
		return new WaveAdtlChunkReader (chunk);
	}



	/**********************************************************************
	Looks up information chunk.

	<p><b>Details:</b>  <code>getInformationChunk</code> searches through this ADTL for the first inner chunk whose cue name is <var>cuename</var> and whose tag is <var>tag</var>.  If found, a <code>RiffChunkReader</code> that reads from the located chunk is returned; otherwise, <code>null</code> is returned.</p>

	<p>This method assumes that the first field in the chunk being sought is <code>dwName</code>, or the name field which identifies the chunk's cue point association.  This assumption is true for
	{@link RIFF_WAVE#LABL LABL},
	{@link RIFF_WAVE#NOTE NOTE},
	{@link RIFF_WAVE#LTXT LTXT}, and
	{@link RIFF_WAVE#FILE FILE} chunks, but it may not be true for other types of other inner chunks that could possibly be present in the ADTL.  The returned <code>RiffChunkReader</code> is positioned immediately after the <code>dwName</code> field when it is returned.</p>

	@param cuename cue name
	@param tag chunk type tag
	@return the chunk, if found
	@exception IOException if an I/O error occurs
	**********************************************************************/

	public RiffChunkReader getInformationChunk (int cuename, int tag) throws IOException
	{
		rcr . seek (4);
		while (rcr . hasMoreData ())
		{
			RiffChunkReader c = rcr . readChunk ();
			if (c . tag != tag)
				continue;
			if (c . readDword () == cuename)
				return c;
		}
		return null;
	}



	private String getStringThingy (int cuename, int tag) throws IOException
	{
		RiffChunkReader c = getInformationChunk (cuename, tag);
		if (c == null)
			return null;
		return c . readString ();
	}



	/**********************************************************************
	Looks up cue point label.

	<p><b>Details:</b>  <code>getLabel</code> returns the label associated with the given cue point name (<var>cuename</var>).  If the label cannot be found, <code>null</code> is returned.</p>

	@param cuename cue name
	@return label
	@exception IOException if an I/O error occurs
	**********************************************************************/

	public String getLabel (int cuename) throws IOException
	{
		return getStringThingy (cuename, RIFF_WAVE.LABL_TAG);
	}



	/**********************************************************************
	Looks up cue point note.

	<p><b>Details:</b>  <code>getNote</code> returns the note associated with the given cue point name (<var>cuename</var>).  If the note cannot be found, <code>null</code> is returned.</p>

	@param cuename cue name
	@return note
	**********************************************************************/

	public String getNote (int cuename) throws IOException
	{
		return getStringThingy (cuename, RIFF_WAVE.NOTE_TAG);
	}



}



