///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  Notice to licensees:                                                     //
//                                                                           //
//  This source code is the exclusive, proprietary intellectual property of  //
//  Sharkysoft (sharkysoft.com).  You may view this source code as a         //
//  supplement to other product documentation, but you may not distribute    //
//  it or use it for any other purpose without written consent from          //
//  Sharkysoft.                                                              //
//                                                                           //
//  You are permitted to modify and recompile this source code, but you may  //
//  not remove this notice.  If you add features to or fix errors in this    //
//  code, please consider sharing your changes with Sharkysoft for possible  //
//  incorporation into future releases of the product.  Thanks!              //
//                                                                           //
//  For more information about Sharkysoft products and services, please      //
//  visit Sharkysoft on the web at                                           //
//                                                                           //
//       http://sharkysoft.com/                                              //
//                                                                           //
//  Thank you for using Lava!                                                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



package lava.riff.wave;



import sharky.LavaMisc;
import java.util.Vector;
import lava.riff.RiffChunkReader;
import java.io.IOException;



/******************************************************************************
Parses cue point list.

<p><b>Details:</b> A <code>WaveCuePointList</code> parses the cue point list (contained in the cue point chunk) of a wave file and provides access to the information contained in the individual cue point structures.  Instances of this class are obtained by calling {@link WaveStreamReader#getCuePoints() <code>WaveStreamReader.getCuePoints</code>}.</p>

<p>In a given .wav file, the cue points may not be stored in any particular order.  If an specific ordering is required, the client should call one of the sorting methods.  Each cue point has, at the very least, an index (i.e., list order), a name (4 byte tag), and a position (i.e., offset into wave sample data).  Cue points often have additional data, such as labels, recorded in a separate ADTL chunk.</p>

@author Sharky
******************************************************************************/

public class WaveCuePointList
{



	private final RiffChunkReader chunk;



	private final WaveStreamReader wsr;



	private final int count;



	private final Vector list = new Vector ();



	WaveCuePointList (RiffChunkReader chunk, WaveStreamReader wsr) throws IOException
	{
		this . chunk = chunk;
		this . wsr = wsr;
		// count: unsigned int
		if (chunk == null)
			count = 0;
		else
			count = chunk . readDword ();
	}



	private WaveCuePointList (WaveCuePoint[] sorted, WaveStreamReader wsr)
	{
		this . chunk = null;
		this . wsr = wsr;
		count = sorted . length;
		list . ensureCapacity (count);
		for (int i = 0; i < count; ++ i)
			list . addElement (sorted [i]);
	}



	private WaveCuePoint getCuePoint (int index) throws IOException
	{
		if (index >= count || index < 0)
			throw new ArrayIndexOutOfBoundsException ("index=" + index);
		if (index >= list . size ())
			list . setSize (index + 1);
		WaveCuePoint wcp = (WaveCuePoint) list . elementAt (index);
		if (wcp == null)
		{
			wcp = new WaveCuePoint (chunk, index);
			list . setElementAt (wcp, index);
		}
		return wcp;
	}



	/**********************************************************************
	Counts cue points.

	<p><b>Details:</b>  <code>countCuePoints</code> determines the total number of cue points in this list and returns it.</p>

	@return number of cue points
	@exception IOException if an I/O error occurs
	**********************************************************************/

	public int countCuePoints () throws IOException
	{
		return count;
	}



	/**********************************************************************
	Returns cue point name.

	<p><b>Details:</b>  <code>getName</code> returns the cue point name of the cue point whose index in the list is <var>index</var>.</p>

	@param index cue point index
	@return cue point name
	@exception IOException if an I/O error occurs
	**********************************************************************/

	public int getName (int index) throws IOException
	{
		return getCuePoint (index) . dwName;
	}



	/**********************************************************************
	Returns cue point position.

	<p><b>Details:</b>  <code>getPosition</code> returns the cue point position of the cue point whose index in the list is <var>index</var>.</p>

	@param index cue point index
	@return cue point position
	@exception IOException if an I/O error occurs
	**********************************************************************/

	public long getPosition (int index) throws IOException
	{
		return getCuePoint (index) . dwPosition;
	}



	/**********************************************************************
	Returns cue point label.

	<p><b>Details:</b>  <code>getLabel</code> returns the cue point label of the cue point whose index in the list is <var>index</var>.</p>

	<p>This method is the inverse of {@see #findLabel(java.lang.String) <code>findLabel</code>}.</p>

	@param index cue point index
	@return cue point label
	@exception IOException if an I/O error occurs
	**********************************************************************/

	public String getLabel (int index) throws IOException
	{
		return wsr . getAdtlChunkReader () . getLabel (getName (index));
	}



	/**********************************************************************
	Searches for cue point label.

	<p><b>Details:</b>  <code>findLabel</code> searches through the list of cue points for the first cue point whose label is <var>label</var>.  If found, the index of the cue point is returned.</p>

	<p>This method is the inverse of {@see #getLabel(int) <code>getLabel</code>}.</p>

	@param label cue point label
	@return cue point index
	@exception IOException if an I/O error occurs
	**********************************************************************/

	public int findLabel (String label) throws IOException
	{
		for (int i = 0; i < count; ++ i)
			if (label . equals (getLabel (i)))
				return i;
		return -1;
	}



	/**********************************************************************
	Sorts list by sample position.

	<p><b>Details:</b>  <code>sortByPosition</code> reads the position values of every cue point in this list, sorts the cue points in order of position, earliest first, and then creates and returns a new <code>WaveCuePointList</code> instance whose cue points are in the sorted order.</p>

	@return sorted list
	@exception IOException if an I/O error occurs
	**********************************************************************/

	public WaveCuePointList sortByPosition () throws IOException
	{
		WaveCuePoint[] sorted = new WaveCuePoint [count];
		for (int i = 0; i < count; ++ i)
			sorted [i] = getCuePoint (i);
		LavaMisc.bubbleSort (sorted, SamplePositionComparator.static_instance);
		return new WaveCuePointList (sorted, wsr);
	}



	/**********************************************************************
	Sorts list by label.

	<p><b>Details:</b>  <code>sortByLabel</code> reads the labels of every cue point in this list, sorts the cue points in alphabetical order, and then creates and returns a new <code>WaveCuePointList</code> instance whose cue points are in the sorted order.</p>

	@return sorted list
	@exception IOException if an I/O error occurs
	**********************************************************************/

	public WaveCuePointList sortByLabel () throws IOException
	{
		WaveCuePoint[] sorted = new WaveCuePoint [count];
		for (int i = 0; i < count; ++ i)
			sorted [i] = getCuePoint (i);
		LavaMisc.bubbleSort (sorted, new LabelComparator (wsr . getAdtlChunkReader ()));
		return new WaveCuePointList (sorted, wsr);
	}



}



