///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  Notice to licensees:                                                     //
//                                                                           //
//  This source code is the exclusive, proprietary intellectual property of  //
//  Sharkysoft (sharkysoft.com).  You may view this source code as a         //
//  supplement to other product documentation, but you may not distribute    //
//  it or use it for any other purpose without written consent from          //
//  Sharkysoft.                                                              //
//                                                                           //
//  You are permitted to modify and recompile this source code, but you may  //
//  not remove this notice.  If you add features to or fix errors in this    //
//  code, please consider sharing your changes with Sharkysoft for possible  //
//  incorporation into future releases of the product.  Thanks!              //
//                                                                           //
//  For more information about Sharkysoft products and services, please      //
//  visit Sharkysoft on the web at                                           //
//                                                                           //
//       http://sharkysoft.com/                                              //
//                                                                           //
//  Thank you for using Lava!                                                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



package lava.riff.wave;



import java.io.IOException;
import lava.io.FileFormatException;
import lava.riff.RiffChunkReader;
import lava.util.BitTwiddler;



/******************************************************************************
Base format chunk.

<p><b>Details:</b> A <code>WaveFormatChunk</code> represents the mandatory portion of a wave format ('fmt') chunk in a RIFF-WAVE file.  All RIFF-WAVE format chunks begin with the fields represented in this class, and then continue with their own, format-<wbr>specific fields.  The mandatory fields are (using "<code>windows.h</code>" notation):</p>

<blockquote>
	<pre>
	struct
	{
	*	WORD  wFormatTag;       // Format category
	*	WORD  wChannels;        // Number of channels
	*	DWORD dwSamplesPerSec;  // Sampling rate
	*	DWORD dwAvgBytesPerSec; // For buffer estimation
	*	WORD  wBlockAlign;      // Data block size
	}
	</pre>
</blockquote>
******************************************************************************/

public abstract class WaveFormatChunk
	implements Cloneable
{



	/**********************************************************************
	Indicates encoding format.

	<p><b>Details:</b> This member is a magic value that indicates the encoding format use for the wave file.  Some possible values include:</p>

	<table border>
		<tr>
			<td>WAVE_FORMAT_PCM</td>
			<td>0x0001</td>
			<td>Microsoft Pulse Code Modulation (PCM)</td>
		</tr>
		<tr>
			<td>IBM_FORMAT_MULAW</td>
			<td>0x0101</td>
			<td>IBM mu-law format</td>
		</tr>
		<tr>
			<td>IBM_FORMAT_ALAW</td>
			<td>0x0102</td>
			<td>IBM a-law format</td>
		</tr>
		<tr>
			<td>IBM_FORMAT_ADPCM</td>
			<td>0x0103</td>
			<td>IBM AVC Adaptive Differential Pulse Code Modulation format</td>
		</tr>
	</table>

	<p>The value of this field also determines the presence of additional fields following the <code>wBlockAlign</code> field.</p>
	**********************************************************************/

	public short wFormatTag;



	/**********************************************************************
	Number of channels.

	<p><b>Details:</b> This member indicates the number of channels represented in the waveform data, i.e., 1 for mono, 2 for stereo, etc.</p>
	**********************************************************************/

	public short wChannels;



	/**********************************************************************
	Full sample rate.

	<p><b>Details:</b> This member indicates the number of samples per second, or the number of times per second each individual channel is sampled.</p>
	**********************************************************************/

	public int dwSamplesPerSec;



	/**********************************************************************
	Average bytes per second.

	<p><b>Details:</b>  This member indicates the average number of bytes per second at which the waveform data will need to be processed.  Playback and recording software can use this estimate when setting up buffers.</p>
	**********************************************************************/

	public int dwAvgBytesPerSec;



	/**********************************************************************
	Block alignment.

	<p><b>Details:</b> This member indicates the block alignment (in bytes) of the waveform data.  Playback software will need to process data blocks of this size.  This value can be used to ensure buffer alignment.</p>
	**********************************************************************/

	public short wBlockAlign;



	/**********************************************************************
	Default constructor.

	<p><b>Details:</b>  This constructor initializes all members to 0.</p>
	**********************************************************************/

	protected WaveFormatChunk ()
	{
	}



	/**********************************************************************
	Initializes from open chunk.

	<p><b>Details:</b> This constructor initializes the members of this <code>WaveFormatChunk</code> with values read from the given RIFF chunk.  It is convenient to use this constructor when parsing wave format chunks.</p>

	@param the RIFF chunk to process as a WaveFormatChunk
	**********************************************************************/

	protected WaveFormatChunk (RiffChunkReader rcr) throws IOException
	{
		rcr . reset ();
		wFormatTag = rcr . readWord ();
		wChannels = rcr . readWord ();
		dwSamplesPerSec = rcr . readDword ();
		dwAvgBytesPerSec = rcr . readDword ();
		wBlockAlign = rcr . readWord ();
	}



	/**********************************************************************
	Validates members.

	<p><b>Details:</b>  This method verifies that the values of this format chunk's fields actually make sense.  If they do not, a <code>FileFormatException</code> is thrown.</p>

	@exception if the field are not valid
	**********************************************************************/

	protected void validateFields () throws FileFormatException
	{
		if (wChannels < 1)
			throw new FileFormatException ("wChannels == " + wChannels);
		if (dwSamplesPerSec < 1)
			throw new FileFormatException ("dwSamplesPerSec == " + dwSamplesPerSec);
		if (dwAvgBytesPerSec < 1)
			throw new FileFormatException ("dwAvgBytesPerSec == " + dwAvgBytesPerSec);
		if (wBlockAlign < 1)
			throw new FileFormatException ("wBlockAlign == " + wBlockAlign);
	}



	/**********************************************************************
	Computes bytes per sample.

	<p><b>Details:</b>  This method returns the number of bytes per sample.  :</p>

	<p><b>Details:</b>  <code>getSampleSize</code> returns the number of bytes required to contain a single, full sample.  The value is determined by considering the number of bits per channel sample and the number of channels, as follows:</p>

	<table border>
		<tr>
			<th>bits per channel sample</th>
			<th>mono</th>
			<th>stereo</th>
		</tr>
		<tr>
			<td>1-8</td>
			<td>1 byte</td>
			<td>2 bytes</td>
		</tr>
		<tr>
			<td>9-16</td>
			<td>2 byte</td>
			<td>4 bytes</td>
		</tr>
		<tr>
			<td>17-24</td>
			<td>3 byte</td>
			<td>6 bytes</td>
		</tr>
		<tr>
			<td>25-32</td>
			<td>4 byte</td>
			<td>8 bytes</td>
		</tr>
	</table>

	<p>These values are calculated without allowing bits from different channel samples to be packed into the same byte.</p>

	@return number of bytes in full sample
	**********************************************************************/

	public abstract int getSampleSize ();



	/**********************************************************************
	Serializes to bytes.

	<p><b>Details:</b>  <code>toBytes</code> serializes this chunk into a string of bytes, as it would appear in a .wav file.</p>

	@return this chunk in byte array form
	**********************************************************************/

	public byte[] toBytes ()
	{
		byte[] bytes = new byte [14];
		BitTwiddler.toBytesLe (wFormatTag      , bytes,  0);
		BitTwiddler.toBytesLe (wChannels       , bytes,  2);
		BitTwiddler.toBytesLe (dwSamplesPerSec , bytes,  4);
		BitTwiddler.toBytesLe (dwAvgBytesPerSec, bytes,  8);
		BitTwiddler.toBytesLe (wBlockAlign     , bytes, 12);
		return bytes;
	}



	/**********************************************************************
	Serializes to string.

	<p><b>Details:</b>  <code>toString</code> produces a string representation of this chunk.  This representation is useful for debugging.</p>

	@return string representation
	**********************************************************************/

	public String toString ()
	{
		return
			"WaveFormatChunk " +
			"(" +
				"wFormatTag: " + wFormatTag + "; " +
				"wChannels: " + wChannels + "; " +
				"dwSamplesPerSec: " + dwSamplesPerSec + "; " +
				"dwAvgBytesPerSec: " + dwAvgBytesPerSec + "; " +
				"wBlockAlign: " + wBlockAlign +
			")";
	}



	/**********************************************************************
	Replicates instance.

	<p><b>Details:</b>  This method returns a perfect replica of this instance.</p>

	@return replica
	**********************************************************************/

	public final Object clone ()
	{
		try
		{
			return super . clone ();
		}
		catch (CloneNotSupportedException e)
		{
			throw new lava.UnreachableCodeException ("WaveFormatChunks must be cloneable");
		}
	}



}



