///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  Notice to licensees:                                                     //
//                                                                           //
//  This source code is the exclusive, proprietary intellectual property of  //
//  Sharkysoft (sharkysoft.com).  You may view this source code as a         //
//  supplement to other product documentation, but you may not distribute    //
//  it or use it for any other purpose without written consent from          //
//  Sharkysoft.                                                              //
//                                                                           //
//  You are permitted to modify and recompile this source code, but you may  //
//  not remove this notice.  If you add features to or fix errors in this    //
//  code, please consider sharing your changes with Sharkysoft for possible  //
//  incorporation into future releases of the product.  Thanks!              //
//                                                                           //
//  For more information about Sharkysoft products and services, please      //
//  visit Sharkysoft on the web at                                           //
//                                                                           //
//       http://sharkysoft.com/                                              //
//                                                                           //
//  Thank you for using Lava!                                                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



package lava.riff.wave;



import lava.array.ArrayCaster;
import lava.konst.DOUBLE;
import lava.konst.FLOAT;



/******************************************************************************
Casts samples from one type to another.

<p><b>Details:</b> <code>WaveSampleConverted</code> casts arrays of samples from one type to another.</p>

<p>The functions in this class have the following general signature:</p>

<blockquote>
<code><var>DEST_TYPE</var>[] toDEST_TYPE (<var>SRC_TYPE</var>[] <var>src</var>, int <var>from</var>, <var>DEST_TYPE</var>[] <var>dest</var>, int <var>to</var>, int <var>amt</var>)</code>
</blockquote>

<p>where</p>

<dl>
	<dt><var>SRC_TYPE</var>
		<dd>is the primitive type of the source array (byte, short, int, float, etc.)
	<dt><var>src</var>
		<dd>is the source array
	<dt><var>from</var>
		<dd>is the starting index in the source array
	<dt><var>DEST_TYPE</var>
		<dd>is the primitive type of the destination array (byte, short, int, float, etc.)
	<dt><var>dest</var>
		<dd>is the destination array
	<dt><var>to</var>
		<dd>is the starting index in the destination array
	<dt><var>amt</var>
		<dd>is the number of elements from the source array to convert
	<dt>the return value
		<dd>is dest, returned for convenience, or a new array, if dest is null
</dl>

<p>If <var>src</var> is <code>null</code>, no action is taken and <code>null</code> is returned.  If <var>dest</var> is <code>null</code>, a new array is created whose length is just long enough to accomodate the requested operation, and that new array is returned.</p>

@since 1998.05.27
@version 1999.09.11
******************************************************************************/

public abstract class WaveSampleConverter
{



	/////////////////
	//             //
	//  toDoubles  //
	//             //
	/////////////////



	public final static double[] toDoubles
	(
		float[] src,
		int from,
		double[] dest,
		int to,
		int amt
	)
	{
		return ArrayCaster.toDoubles (src, from, dest, to, amt);
	}



	public static double[] toDoubles
	(
		long[] src,
		int from,
		double[] dest,
		int to,
		int amt
	)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new double [to + amt];
		for (int i = 0; i < amt; ++ i)
			dest [to + i] = src [from + i] / DOUBLE.MAX_SLONG;
		return dest;
	}



	public static double[] toDoubles
	(
		int[] src,
		int from,
		double[] dest,
		int to,
		int amt
	)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new double [to + amt];
		for (int i = 0; i < amt; ++ i)
			dest [to + i] = src [from + i] / DOUBLE.MAX_SINT;
		return dest;
	}



	public static double[] toDoubles
	(
		short[] src,
		int from,
		double[] dest,
		int to,
		int amt
	)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new double [to + amt];
		for (int i = 0; i < amt; ++ i)
			dest [to + i] = src [from + i] / DOUBLE.MAX_SSHORT;
		return dest;
	}



	public static double[] toDoubles
	(
		byte[] src,
		int from,
		double[] dest,
		int to,
		int amt
	)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new double [to + amt];
		for (int i = 0; i < amt; ++ i)
			dest [to + i] = ((src [from + i] & 0xff) - 128) / DOUBLE.MAX_SBYTE;
		return dest;
	}



	////////////////
	//            //
	//  toFloats  //
	//            //
	////////////////



	public final static float[] toFloats
	(
		double[] src,
		int from,
		float[] dest,
		int to,
		int amt
	)
	{
		return ArrayCaster.toFloats (src, from, dest, to, amt);
	}



	public static float[] toFloats
	(
		long[] src,
		int from,
		float[] dest,
		int to,
		int amt
	)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new float [to + amt];
		for (int i = 0; i < amt; ++ i)
			dest [to + i] = src [from + i] / FLOAT.MAX_SLONG;
		return dest;
	}



	public static float[] toFloats
	(
		int[] src,
		int from,
		float[] dest,
		int to,
		int amt
	)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new float [to + amt];
		for (int i = 0; i < amt; ++ i)
			dest [to + i] = src [from + i] / FLOAT.MAX_SINT;
		return dest;
	}



	public static float[] toFloats
	(
		short[] src,
		int from,
		float[] dest,
		int to,
		int amt
	)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new float [to + amt];
		for (int i = 0; i < amt; ++ i)
			dest [to + i] = src [from + i] / FLOAT.MAX_SSHORT;
		return dest;
	}



	public static float[] toFloats
	(
		byte[] src,
		int from,
		float[] dest,
		int to,
		int amt
	)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new float [to + amt];
		for (int i = 0; i < amt; ++ i)
			dest [to + i] = ((src [from + i] & 0xff) - 128) / FLOAT.MAX_SBYTE;
		return dest;
	}



	///////////////
	//           //
	//  toLongs  //
	//           //
	///////////////



	public static long[] toLongs
	(
		double[] src,
		int from,
		long[] dest,
		int to,
		int amt
	)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new long [to + amt];
		for (int i = 0; i < amt; ++ i)
			dest [to + i] = (long) (src [from + i] * DOUBLE.MAX_SLONG);
		return dest;
	}



	public static long[] toLongs
	(
		float[] src,
		int from,
		long[] dest,
		int to,
		int amt
	)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new long [to + amt];
		for (int i = 0; i < amt; ++ i)
			dest [to + i] = (long) (src [from + i] * FLOAT.MAX_SLONG);
		return dest;
	}



	public static long[] toLongs
	(
		int[] src,
		int from,
		long[] dest,
		int to,
		int amt
	)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new long [to + amt];
		for (int i = 0; i < amt; ++ i)
			dest [to + i] = (long) src [from + i] << 4 * 8;
		return dest;
	}



	public static long[] toLongs
	(
		short[] src,
		int from,
		long[] dest,
		int to,
		int amt
	)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new long [to + amt];
		for (int i = 0; i < amt; ++ i)
			dest [to + i] = (long) src [from + i] << 6 * 8;
		return dest;
	}



	public static long[] toLongs
	(
		byte[] src,
		int from,
		long[] dest,
		int to,
		int amt
	)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new long [to + amt];
		for (int i = 0; i < amt; ++ i)
			dest [to + i] = (long) ((src [from + i] & 0xff) - 128) << 7 * 8;
		return dest;
	}



	//////////////
	//          //
	//  toInts  //
	//          //
	//////////////



	public static int[] toInts
	(
		double[] src,
		int from,
		int[] dest,
		int to,
		int amt
	)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new int [to + amt];
		for (int i = 0; i < amt; ++ i)
			dest [to + i] = (int) (src [from + i] * DOUBLE.MAX_SINT);
		return dest;
	}



	public static int[] toInts
	(
		float[] src,
		int from,
		int[] dest,
		int to,
		int amt
	)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new int [to + amt];
		for (int i = 0; i < amt; ++ i)
			dest [to + i] = (int) (src [from + i] * FLOAT.MAX_SINT);
		return dest;
	}



	public final static int[] toInts
	(
		long[] src,
		int from,
		int[] dest,
		int to,
		int amt
	)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new int [to + amt];
		for (int i = 0; i < amt; ++ i)
			dest [to + i] = (int) (src [from + i] >> 4 * 8);
		return dest;
	}



	public static int[] toInts
	(
		short[] src,
		int from,
		int[] dest,
		int to,
		int amt
	)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new int [to + amt];
		for (int i = 0; i < amt; ++ i)
			dest [to + i] = src [from + i] << 2 * 8;
		return dest;
	}



	public static int[] toInts
	(
		byte[] src,
		int from,
		int[] dest,
		int to,
		int amt
	)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new int [to + amt];
		for (int i = 0; i < amt; ++ i)
			dest [to + i] = (src [from + i] & 0xff) - 128 << 3 * 8;
		return dest;
	}



	////////////////
	//            //
	//  toShorts  //
	//            //
	////////////////



	public static short[] toShorts
	(
		double[] src,
		int from,
		short[] dest,
		int to,
		int amt
	)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new short [to + amt];
		for (int i = 0; i < amt; ++ i)
			dest [to + i] = (short) (src [from + i] * DOUBLE.MAX_SSHORT);
		return dest;
	}



	public static short[] toShorts
	(
		float[] src,
		int from,
		short[] dest,
		int to,
		int amt
	)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new short [to + amt];
		for (int i = 0; i < amt; ++ i)
			dest [to + i] = (short) (src [from + i] * FLOAT.MAX_SSHORT);
		return dest;
	}



	public final static short[] toShorts
	(
		long[] src,
		int from,
		short[] dest,
		int to,
		int amt
	)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new short [to + amt];
		for (int i = 0; i < amt; ++ i)
			dest [to + i] = (short) (src [from + i] >> 6 * 8);
		return dest;
	}



	public final static short[] toShorts
	(
		int[] src,
		int from,
		short[] dest,
		int to,
		int amt
	)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new short [to + amt];
		for (int i = 0; i < amt; ++ i)
			dest [to + i] = (short) (src [from + i] >> 2 * 8);
		return dest;
	}



	public static short[] toShorts
	(
		byte[] src,
		int from,
		short[] dest,
		int to,
		int amt
	)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new short [to + amt];
		for (int i = 0; i < amt; ++ i)
			dest [to + i] = (short) ((src [from + i] & 0xff) - 128 << 8);
		return dest;
	}



	///////////////
	//           //
	//  toBytes  //
	//           //
	///////////////



	public static byte[] toBytes
	(
		double[] src,
		int from,
		byte[] dest,
		int to,
		int amt
	)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new byte [to + amt];
		for (int i = 0; i < amt; ++ i)
			dest [to + i] = (byte) ((int) (src [from + i] * DOUBLE.MAX_SBYTE) + 128);
		return dest;
	}



	public static byte[] toBytes
	(
		float[] src,
		int from,
		byte[] dest,
		int to,
		int amt
	)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new byte [to + amt];
		for (int i = 0; i < amt; ++ i)
			dest [to + i] = (byte) ((int) (src [from + i] * FLOAT.MAX_SBYTE) + 128);
		return dest;
	}



	public static byte[] toBytes
	(
		long[] src,
		int from,
		byte[] dest,
		int to,
		int amt
	)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new byte [to + amt];
		for (int i = 0; i < amt; ++ i)
			dest [to + i] = (byte) ((int) (src [from + i] >> 7 * 8) + 128);
		return dest;
	}



	public final static byte[] toBytes
	(
		int[] src,
		int from,
		byte[] dest,
		int to,
		int amt
	)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new byte [to + amt];
		for (int i = 0; i < amt; ++ i)
			dest [to + i] = (byte) ((src [from + i] >> 3 * 8) + 128);
		return dest;
	}



	public final static byte[] toBytes
	(
		short[] src,
		int from,
		byte[] dest,
		int to,
		int amt
	)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new byte [to + amt];
		for (int i = 0; i < amt; ++ i)
			dest [to + i] = (byte) ((src [from + i] >> 8) + 128);
		return dest;
	}



	///////////////////////////////
	//                           //
	//  triple-byte conversions  //
	//                           //
	///////////////////////////////



	// amt == number of conversions to make
	static byte[] toBytes3 (byte[] src, int from, byte[] dest, int to, int amt)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new byte [to + amt];
		for (int i = 0; i < amt; ++ i)
			dest [to + i] = src [from + 3 * i + 2];
		return dest;
	}



	static short[] toShorts3 (byte[] src, int from, short[] dest, int to, int amt)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new short [to + amt];
		for (int i = 0; i < amt; ++ i)
		{
			final int frame = from + 3 * i;
			dest [to + i] = (short) (src [frame + 2] << 8 | src [frame + 1] & 0xff);
		}
		return dest;
	}



	static int[] toInts3 (byte[] src, int from, int[] dest, int to, int amt)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new int [to + amt];
		for (int i = 0; i < amt; ++ i)
		{
			final int frame = from + 3 * i;
			dest [to + i] =
				src [frame + 2] << 3 * 8
			|	src [frame + 1] << 2 * 8 & 0xff0000
			|	src [frame    ] <<     8 &   0xff00;
		}
		return dest;
	}



	static long[] toLongs3 (byte[] src, int from, long[] dest, int to, int amt)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new long [to + amt];
		for (int i = 0; i < amt; ++ i)
		{
			final int frame = from + 3 * i;
			dest [to + i] =
				(long)
				(
					src [frame + 2] << 3 * 8
				|	src [frame + 1] << 2 * 8 & 0xff0000
				|	src [frame    ] <<     8 &   0xff00
				)
			<<	4 * 8;
		}
		return dest;
	}



	static float[] toFloats3 (byte[] src, int from, float[] dest, int to, int amt)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new float [to + amt];
		for (int i = 0; i < amt; ++ i)
		{
			final int frame = from + 3 * i;
			dest [to + i] =
				(
					src [frame + 2] << 2 * 8
				|	src [frame + 1] <<     8 & 0xff00
				|	src [frame    ]          &   0xff
				)
			/	(float) 0x7fffff;
		}
		return dest;
	}



	static double[] toDoubles3 (byte[] src, int from, double[] dest, int to, int amt)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new double [to + amt];
		for (int i = 0; i < amt; ++ i)
		{
			final int frame = from + 3 * i;
			dest [to + i] =
				(
					src [frame + 2] << 2 * 8
				|	src [frame + 1] <<     8 & 0xff00
				|	src [frame    ]          &   0xff
				)
			/	(double) 0x7fffff;
		}
		return dest;
	}



	static byte[] to3Bytes (int[] src, int from, byte[] dest, int to, int amt)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new byte [to + amt * 3];
		for (int i = 0; i < amt; ++ i)
		{
			final int frame = to + 3 * i;
			final int value = src [from + i];
			dest [frame    ] = (byte) (value >>     8);
			dest [frame + 1] = (byte) (value >> 2 * 8);
			dest [frame + 2] = (byte) (value >> 3 * 8);
		}
		return dest;
	}



}



