///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  Notice to licensees:                                                     //
//                                                                           //
//  This source code is the exclusive, proprietary intellectual property of  //
//  Sharkysoft (sharkysoft.com).  You may view this source code as a         //
//  supplement to other product documentation, but you may not distribute    //
//  it or use it for any other purpose without written consent from          //
//  Sharkysoft.                                                              //
//                                                                           //
//  You are permitted to modify and recompile this source code, but you may  //
//  not remove this notice.  If you add features to or fix errors in this    //
//  code, please consider sharing your changes with Sharkysoft for possible  //
//  incorporation into future releases of the product.  Thanks!              //
//                                                                           //
//  For more information about Sharkysoft products and services, please      //
//  visit Sharkysoft on the web at                                           //
//                                                                           //
//       http://sharkysoft.com/                                              //
//                                                                           //
//  Thank you for using Lava!                                                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



package lava.riff.wave;



import lava.konst.LONG;
import lava.konst.INT;



/******************************************************************************
Rounds wave sample data.

<p><b>Details:</b> <code>WaveSampleRounder</code> is a collection of functions that round sample arrays to a desired precision.</p>

<p>The parameters for each of these methods follow the same pattern as described in the header for class {@link WaveSampleConverter <code>WaveSampleConverter</code>}, with the exception that there is an additional parameter that specifies the rounding precision.</p>

@author Sharky
******************************************************************************/

public class WaveSampleRounder
{



	/**********************************************************************
	Rounds values to requested precision.

	<p><b>Details:</b>  <code>round</code> modifies the sample data so that each value is rounded to the specified number of bits.  Rounding is accomplished by setting the least significant bits to 0.</p>

	@param src source array
	@param from starting index in src
	@param dest destination array
	@param to starting index in dest
	@param amt number of values to clip
	@param bits number of bits of precision
	@return destination array
	**********************************************************************/

	public static long[] round
	(
		long[] src,
		int from,
		long[] dest,
		int to,
		int amt,
		int bits
	)
	{
		if (bits == 64 && src == dest && from == to)
			return dest;
		if (bits < 0 || bits > 64)
			throw new IllegalArgumentException ("bits==" + bits);
		if (src == null)
			return null;
		if (dest == null)
			dest = new long [to + amt];
		final long mask = LONG.MAX_ULONG << 64 - bits;
		for (int i = 0; i < amt; ++ i)
			dest [to + i] = src [from + i] & mask;
		return dest;
	}



	/**********************************************************************
	Rounds values to requested precision.

	<p><b>Details:</b>  <code>round</code> modifies the sample data so that each value is rounded to the specified number of bits.  Rounding is accomplished by setting the least significant bits to 0.</p>

	@param src source array
	@param from starting index in src
	@param dest destination array
	@param to starting index in dest
	@param amt number of values to clip
	@param bits number of bits of precision
	@return destination array
	**********************************************************************/

	public static int[] round
	(
		int[] src,
		int from,
		int[] dest,
		int to,
		int amt,
		int bits
	)
	{
		if (bits == 32 && src == dest && from == to)
			return dest;
		if (bits < 0 || bits > 32)
			throw new IllegalArgumentException ("bits==" + bits);
		if (src == null)
			return null;
		if (dest == null)
			dest = new int [to + amt];
		final int mask = INT.MAX_UINT << 32 - bits;
		for (int i = 0; i < amt; ++ i)
			dest [to + i] = src [from + i] & mask;
		return dest;
	}



	/**********************************************************************
	Rounds values to requested precision.

	<p><b>Details:</b>  <code>round</code> modifies the sample data so that each value is rounded to the specified number of bits.  Rounding is accomplished by setting the least significant bits to 0.</p>

	@param src source array
	@param from starting index in src
	@param dest destination array
	@param to starting index in dest
	@param amt number of values to clip
	@param bits number of bits of precision
	@return destination array
	**********************************************************************/

	public static short[] round
	(
		short[] src,
		int from,
		short[] dest,
		int to,
		int amt,
		int bits
	)
	{
		if (bits == 16 && src == dest && from == to)
			return dest;
		if (bits < 0 || bits > 16)
			throw new IllegalArgumentException ("bits==" + bits);
		if (src == null)
			return null;
		if (dest == null)
			dest = new short [to + amt];
		final int mask = INT.MAX_UINT << 16 - bits;
		for (int i = 0; i < amt; ++ i)
			dest [to + i] = (short) (src [from + i] & mask);
		return dest;
	}



	/**********************************************************************
	Rounds values to requested precision.

	<p><b>Details:</b>  <code>round</code> modifies the sample data so that each value is rounded to the specified number of bits.  Rounding is accomplished by setting the least significant bits to 0.</p>

	@param src source array
	@param from starting index in src
	@param dest destination array
	@param to starting index in dest
	@param amt number of values to clip
	@param bits number of bits of precision
	@return destination array
	**********************************************************************/

	public static byte[] round
	(
		byte[] src,
		int from,
		byte[] dest,
		int to,
		int amt,
		int bits
	)
	{
		if (bits == 8 && src == dest && from == to)
			return dest;
		if (bits < 0 || bits > 8)
			throw new IllegalArgumentException ("bits==" + bits);
		if (src == null)
			return null;
		if (dest == null)
			dest = new byte [to + amt];
		final int mask = INT.MAX_UINT << 8 - bits;
		for (int i = 0; i < amt; ++ i)
			dest [to + i] = (byte) (src [from + i] & mask);
		return dest;
	}



}



