///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  Notice to licensees:                                                     //
//                                                                           //
//  This source code is the exclusive, proprietary intellectual property of  //
//  Sharkysoft (sharkysoft.com).  You may view this source code as a         //
//  supplement to other product documentation, but you may not distribute    //
//  it or use it for any other purpose without written consent from          //
//  Sharkysoft.                                                              //
//                                                                           //
//  You are permitted to modify and recompile this source code, but you may  //
//  not remove this notice.  If you add features to or fix errors in this    //
//  code, please consider sharing your changes with Sharkysoft for possible  //
//  incorporation into future releases of the product.  Thanks!              //
//                                                                           //
//  For more information about Sharkysoft products and services, please      //
//  visit Sharkysoft on the web at                                           //
//                                                                           //
//       http://sharkysoft.com/                                              //
//                                                                           //
//  Thank you for using Lava!                                                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



package lava.riff.wave;



//import java.io.EOFException;
import java.io.IOException;
import java.util.Vector;
import lava.riff.RiffChunkReader;
import lava.riff.RiffStreamReader;
import lava.riff.IRiffInput;
import lava.io.FileFormatException;



/******************************************************************************
Base class for all .wav stream decoders.

<p><b>Details:</b> A <code>WaveStreamReader</code> reads and returns information, including format data and wave samples, from a RIFF-wave file.  <code>WaveStreamReader</code> is the abstract superclass of all wave decoders, such as <code>PcmWaveFileReader</code>.</p>
******************************************************************************/

public abstract class WaveStreamReader
{



	/**********************************************************************
	Parses RIFF stream.

	<p><b>Details:</b>  This member is the <code>RiffStreamReader</code> used by this <code>WaveStreamReader</code> uses to parse the wave stream, which is a special type of RIFF stream.</p>
	**********************************************************************/

	protected final RiffStreamReader riff;



	/**********************************************************************
	Parsed format chunk.

	<p><b>Details:</b>  <var>fmt</var> represents the parsed format chunk from this wave stream.  This member must be set by the subclass, or some methods in this superclass which require access to the chunk will throw an exception.</p>
	**********************************************************************/

	protected WaveFormatChunk fmt;



	private final Vector chunks = new Vector ();



	/**********************************************************************
	Sets wave stream source.

	<p><b>Details:</b>  This constructor sets the stream that this <code>WaveStreamReader</code> will decode.</p>

	@param raise the stream
	**********************************************************************/

	protected WaveStreamReader (IRiffInput rais) throws IOException
	{
		riff = new RiffStreamReader (rais);
		riff . reset ();
		int tag = riff . readTag ();
		if (tag != RIFF_WAVE.WAVE_TAG)
			throw new FileFormatException ("rais is not a valid wave stream because the 'WAVE' tag is missing.");
		riff . freeHeader ();
		riff . free (0, 4);
	}



	/**********************************************************************
	Returns chunk reader by index.

	<p><b>Details:</b>  This method returns a <code>RiffChunkReader</code> for the <var>n</var><sup>th</sup> chunk in this wave stream (where the first chunk has index 0).  Beware that if the requested chunk hasn't been reached yet, the RIFF source may be read until the chunk is found or until the EOS is reached.  This may place a strain on the buffering mechanism of the underlying <code>IRiffInput</code>.  <code>null</code> is returned if the chunk doesn't exist.</p>

	@param n the chunk number
	@return the chunk reader
	**********************************************************************/

	protected RiffChunkReader getChunkReader (int n) throws IOException
	{
		if (n < 0)
			return null;
		while (chunks . size () <= n)
		{
			if (! riff . hasMoreData ())
				return null;
			RiffChunkReader rcr = riff . readChunk ();
			rcr . freeHeader ();
			chunks . addElement (rcr);
		}
		return (RiffChunkReader) chunks . elementAt (n);
	}



	/**********************************************************************
	Returns chunk reader by tag.

	<p><b>Details:</b>  This method searchs, starting with the <var>from</var><sup>th</sup> chunk, for a chunk whose tag (chunk type identifier) is equal to <var>tag</var>.  Beware that if the chunk being sought hasn't been reached yet, the RIFF source may be read until the chunk is found, or until the RIFF source has been exhausted, possibly placing a strain on the buffering mechanism of the underlying <code>IRiffInput</code>.  <code>null</code> is returned if the chunk can't be found.</p>

	@param from the starting chunk
	@param tag the chunk type identifier
	@return the chunk reader
	**********************************************************************/

	protected int findChunk (int from, int tag) throws IOException
	{
		while (true)
		{
			RiffChunkReader rcr = getChunkReader (from);
			if (rcr == null)
				return -1;
			if (rcr . tag == tag)
				return from;
			++ from;
		}
	}



	private WaveCuePointList cue_points = null;



	/**********************************************************************
	Parses cue points chunk.

	<p><b>Details:</b>  This method (partially) parses the wave stream's cue point list, if there is one, and returns an object which can be used to query specific details from the list.  If this wave stream has no cue point list, this method returns <code>null</code>.</p>

	@return the cue points list
	**********************************************************************/

	public synchronized WaveCuePointList getCuePoints () throws IOException
	{
		if (cue_points == null)
			cue_points = new WaveCuePointList (getChunkReader (findChunk (0, RIFF_WAVE.CUE_TAG)), this);
		return cue_points;
	}



	/**********************************************************************
	Returns ADTL chunk parser.

	<p><b>Details:</b>  This method returns a <code>WaveAdtlChunkReader</code>, or an object capable of parsing this stream's ADTL (associated data list) chunk.  The ADTL chunk can contain a wealth of information abou this stream's cue points.  If this wave stream has no ADTL chunk, this method returns <code>null</code>.</p>

	@return the WaveAdtlChunkReader
	**********************************************************************/

	public WaveAdtlChunkReader getAdtlChunkReader () throws IOException
	{
		int p = 0;
		while (true)
		{
			p = findChunk (p, RIFF_WAVE.LIST_TAG);
			if (p < 0)
				return null;
			RiffChunkReader rcr = getChunkReader (p);
			WaveAdtlChunkReader adtl = WaveAdtlChunkReader.create (rcr);
			if (adtl != null)
				return adtl;
			++ p;
		}
	}



	/**********************************************************************
	Returns format chunk.

	<p><b>Details:</b>  This method returns an object representing the parsed format chunk.  The returned object is a mutable clone of the original.</p>

	@return the format chunk
	**********************************************************************/

	public WaveFormatChunk getFormatChunk ()
	{
		return (WaveFormatChunk) fmt . clone ();
	}



	/**********************************************************************
	Returns number of channels.

	<p><b>Details:</b>  This method returns the number of audio channels represented by this wave stream.  Typically, 1 indicates monoaural sound and 2 indicates stereo sound.</p>

	@return channel count
	**********************************************************************/

	public int countChannels ()
	{
		return fmt . wChannels;
	}



	/**********************************************************************
	Returns number of bits per channel sample.

	<p><b>Details:</b>  This method returns the number of bits per sample per channel.</p>

	@return number of bits per sample channel sample.
	**********************************************************************/

	public abstract int getBitsPerChannelSample ();



	/**********************************************************************
	Returns number of samples.

	<p><b>Details:</b>  This method returns the number of samples contained in this wave stream.  This value devided by the sample rate is the duration of the sound.</p>

	@return number of samples
	**********************************************************************/

	public abstract long countSamples () throws IOException;



	/**********************************************************************
	Reads and decodes samples.

	<p><b>Details:</b>  This method reads sample data from the source file and returns the samples in a <a href=package-summary.html#sample_arrays>sample array</a>.  Exactly <var>amount</var> samples are read, beginning with the <var>when</var><sup>th</sup> sample in the sample stream (where 0 is the index of the first sample).  The samples are copied into the given array (<var>dest</var>).  If that array is not the correct type for the sample resolution of this wave stream, the samples are automatically converted to the resolution required by the array.  (See the <a href=package-summary.html>package header</a> for more details.)</p>

	<p>If <var>dest</var> is <code>null</code>, an array of the most natural type for this stream is automatically created, and its length is set just large enough to accomodate the request.  The new sample array is then treated exactly as if it had been provided as the original <var>dest</var> parameter.</p>

	<p>The first sample written into <var>dest</var> is written into <var>dest</var> [<var>dest_from</var>].  Exactly <var>duration</var> * <var>wChannels</var> values are decoded and written into the array, where <var>wChannels</var> is the number of channels as set in the format chunk.</p>

	@param when sample position
	@param dest destination array
	@param dest_from starting index in destination array
	@param duration number of samples to read
	@return the samples
	@exception IOException if an I/O error occurs
	**********************************************************************/

	public abstract Object readSamples (long when, Object dest, int dest_from, int duration) throws IOException;



	/**********************************************************************
	Releases samples from buffer.

	<p><b>Details:</b>  This method informs this wave stream reader that the client will no longer requires access to the given range of samples.  Calling this method allows this implementation to send buffering hints to the underlying .wav file input source, which may be buffering the data stream.</p>

	@param when beginning sample index
	@param duration number of samples to release
	@exception if an I/O error occurs

	@see lava.riff.IRiffInput#free(long, long)
	**********************************************************************/

	public abstract void freeSamples (long when, int duration) throws IOException;



	/**********************************************************************
	Closes wave stream.

	<p><b>Details:</b>  This method closes the RIFF source that this <code>WaveStreamReader</code> was processing.  Calls to other methods in the class may fail after calling <code>close</code>.</p>
	**********************************************************************/

	public void close () throws IOException
	{
		riff . close ();
	}



	/**********************************************************************
	Returns size of individual channel sample.

	<p><b>Details:</b>  <code>getChannelSampleSize</code> returns the size of the data type used by <code>readSamples</code> to store a single channel sample.  The value returned by this method indicates the type of array most naturally preferred by this instance in calls to <code>readSamples</code>.</p>

	<table>
		<tr>
			<th>channel sample size</th>
			<th>value returned</th>
			<th>array type</th>
		</tr>
		<tr>
			<td>1-8 bits</td>
			<td>1</td>
			<td>byte</td>
		</tr>
		<tr>
			<td>9-16 bits</td>
			<td>2</td>
			<td>short</td>
		</tr>
		<tr>
			<td>17-32 bits</td>
			<td>4</td>
			<td>int</td>
		</tr>
	</table>

	@return size of array type

	@since 2000.09.28
	**********************************************************************/

	public abstract int getChannelSampleSize () throws IOException;



}



