///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  Notice to licensees:                                                     //
//                                                                           //
//  This source code is the exclusive, proprietary intellectual property of  //
//  Sharkysoft (sharkysoft.com).  You may view this source code as a         //
//  supplement to other product documentation, but you may not distribute    //
//  it or use it for any other purpose without written consent from          //
//  Sharkysoft.                                                              //
//                                                                           //
//  You are permitted to modify and recompile this source code, but you may  //
//  not remove this notice.  If you add features to or fix errors in this    //
//  code, please consider sharing your changes with Sharkysoft for possible  //
//  incorporation into future releases of the product.  Thanks!              //
//                                                                           //
//  For more information about Sharkysoft products and services, please      //
//  visit Sharkysoft on the web at                                           //
//                                                                           //
//       http://sharkysoft.com/                                              //
//                                                                           //
//  Thank you for using Lava!                                                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



package lava.riff.wave;



import lava.riff.IRiffOutput;
import lava.riff.RiffStreamWriter;
import java.io.IOException;



/******************************************************************************
Base class for .wav file encoders.

<p><b>Details:</b>  <code>WaveStreamWriter</code> is the generic base class for wave file encoders.  This class handles basic wave stream writing tasks that are common to all wave writers regardless of the encoding format.</p>
******************************************************************************/

public abstract class WaveStreamWriter
{



	/**********************************************************************
	RIFF file generator.

	<p><b>Details:</b>  <var>rsw</var> is the underlying <code>RiffStreamWriter</code> used to generate this .wav file, which is a special kind of RIFF file.</p>
	**********************************************************************/

	protected RiffStreamWriter rsw;



	/**********************************************************************
	Outputs format chunk.

	<p><b>Details:</b>  This constructor opens a new <code>RiffStreamWriter</code> and writes the given wave format chunk (<var>fmt_chunk</var>), in preparation for generating the rest of the .wav file.</p>

	@param raos the destination output stream
	@param fmt_chunk the format chunk
	**********************************************************************/

	public WaveStreamWriter (IRiffOutput raos, WaveFormatChunk fmt_chunk) throws IOException
	{
		rsw = new RiffStreamWriter (raos, 0);
		rsw . writeTag (RIFF_WAVE.WAVE_TAG);
		byte[] chunk_bytes = fmt_chunk . toBytes ();
		rsw . beginChunk (RIFF_WAVE.FMT_TAG, chunk_bytes . length);
		rsw . writeBytes (chunk_bytes, 0, chunk_bytes . length);
		rsw . endChunk ();
	}



	/**********************************************************************
	Opens wave sample data chunk.

	<p><b>Details:</b>  <code>beginSamples</code> prepares this <code>WaveStreamWriter</code> to begin recording wave sample data.  This method must be called before <code>writeSamples</code> can be called.</p>

	<p>If the number of (full) samples to be written is known ahead of time, that value should be provided in the <var>duration</var> parameter; otherwise, set this parameter to 0.  Providing the <var>duration</var> value ahead of time allows the RIFF writer to complete the header that must appear before the sample data.  Otherwise, the RIFF writer will need to seek backward in the file to complete the header after <code>endSamples</code> is called, which can strain the underlying buffering mechanisms if the data isn't being written to a standard random access file.</p>

	@param duration number of samples
	@exception IOException if an I/O error occurs
	**********************************************************************/

	public abstract void beginSamples (long duration) throws IOException;



	/**********************************************************************
	Encodes and writes samples.

	<p><b>Details:</b>  <code>writeSamples</code> writes the given samples (in <var>samples</var>) into the wave file's wave data chunk.  The sample values are given in a sample array, which is described in the package header.  If the sample resolution <a href=package-summary.html#sample_arrays>implied</a> by the array's primitive type is not the sample resolution of this wave file (as set in the format chunk), this method will automatically convert the samples before writing them.</p>

	<p>The first sample in the given array is taken from <var>samples</var> [<var>samples_from</var>].  Exactly <var>duration</var> * <var>wChannels</var> values are encoded and written to the stream, where <var>wChannels</var> is the number of channels as set in the format chunk.</p>

	@param samples sample array
	@param samples_from starting offset into array
	@param duration number of samples to process
	@exception IOException if an I/O error occurs
	**********************************************************************/

	public abstract void writeSamples (Object samples, int samples_from, int duration) throws IOException;



	/**********************************************************************
	Closes wave data chunk.

	<p><b>Details:</b>  Call <code>endSamples</code> when there are no more wave samples to encode and write to the wave stream.  Because this method causes the underlying RIFF stream generator to close the wave data chunk, no more samples can be written after this method is called.</p>

	@exception IOException if an I/O error occurs
	**********************************************************************/

	public abstract void endSamples () throws IOException;



	/**********************************************************************
	Closes stream.

	<p><b>Details:</b>  This method closes all open chunks in the wave stream and completes the stream.  The close method of the underlying RiffStreamWriter is also called, which in turn calls the close method of its underlying IRiffOutput.</p>

	@exception IOException if an I/O error occurs
	**********************************************************************/

	public abstract void close () throws IOException;



}



