///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  Notice to licensees:                                                     //
//                                                                           //
//  This source code is the exclusive, proprietary intellectual property of  //
//  Sharkysoft (sharkysoft.com).  You may view this source code as a         //
//  supplement to other product documentation, but you may not distribute    //
//  it or use it for any other purpose without written consent from          //
//  Sharkysoft.                                                              //
//                                                                           //
//  You are permitted to modify and recompile this source code, but you may  //
//  not remove this notice.  If you add features to or fix errors in this    //
//  code, please consider sharing your changes with Sharkysoft for possible  //
//  incorporation into future releases of the product.  Thanks!              //
//                                                                           //
//  For more information about Sharkysoft products and services, please      //
//  visit Sharkysoft on the web at                                           //
//                                                                           //
//       http://sharkysoft.com/                                              //
//                                                                           //
//  Thank you for using Lava!                                                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



package lava.util;



/******************************************************************************
Bit-twiddling operations.

<p><code>BitTwiddler</code> is a collection of static methods that perform bit-twiddling operations.</p>

@version 1999.08.08
******************************************************************************/

public class BitTwiddler
{



	///////////////////////////////////////
	//                                   //
	//  byte[]-to-primitive conversions  //
	//                                   //
	///////////////////////////////////////



	/**********************************************************************
	Converts big endian bytes into int.

	<p><b>Details:</b>  This method converts big endian ordered <code>byte</code>s into an <code>int</code>.  <var>offset</var> is the array index of the first byte.</p>

	@param b the byte array
	@param the array offset (<b>default value:</b> 0)
	@return the int

	@since 1999.08.01
	**********************************************************************/

	public static final int toIntBe (byte[] b, int offset)
	{
		return
			(b [offset + 0] & 0xff) << (3 * 8)
		|	(b [offset + 1] & 0xff) << (2 * 8)
		|	(b [offset + 2] & 0xff) << (1 * 8)
		|	 b [offset + 3] & 0xff            ;
	}



	/**********************************************************************
	Shorthand for toIntBe (b, 0).
	@see #toIntBe(byte[], int)
	@version 1999.08.01
	@since 1998.11.19
	**********************************************************************/

	public static final int toIntBe (byte[] b)
	{
		return toIntBe (b, 0);
	}



	/**********************************************************************
	Converts little endian bytes into int.

	<p><b>Details:</b>  This method converts little endian ordered <code>byte</code>s into an <code>int</code>.  <var>offset</var> is the array index of the first byte.</p>

	@param b the byte array
	@param the array offset (<b>default value:</b> 0)
	@return the int

	@since 1999.08.01
	**********************************************************************/

	public static final int toIntLe (byte [] b, int offset)
	{
		return
			(b [offset + 3] & 0xff) << (3 * 8)
		|	(b [offset + 2] & 0xff) << (2 * 8)
		|	(b [offset + 1] & 0xff) << (1 * 8)
		|	 b [offset + 0] & 0xff            ;
	}



	/**********************************************************************
	Converts little endian bytes into short.

	<p><b>Details:</b>  This method converts little endian ordered <code>byte</code>s into a <code>short</code>.  <var>offset</var> is the array index of the first byte.</p>

	@param b the byte array
	@param the array offset (<b>default value:</b> 0)
	@return the short

	@since 1999.08.01
	**********************************************************************/

	public static final short toShortLe (byte [] b, int offset)
	{
		return (short) (b [offset + 1] << 8 | b [offset] & 0xff);
	}



	///////////////////////////////////////
	//                                   //
	//  primitive-to-byte[] conversions  //
	//                                   //
	///////////////////////////////////////



	/**********************************************************************
	Breaks int into big endian bytes.

	<p><b>Details:</b>  This method breaks an <code>int</code> into 4 <code>byte</code>s and stores them in the given array, at the given index, using big endian byte ordering.</p>

	@param i the int
	@param dest the destination array
	@param offset the array index
	@return dest
	**********************************************************************/

	public static final byte[] toBytesBe (int i, byte[] dest, int offset)
	{
		dest [offset + 0] = (byte) (i >> 3 * 8);
		dest [offset + 1] = (byte) (i >> 2 * 8);
		dest [offset + 2] = (byte) (i >> 1 * 8);
		dest [offset + 3] = (byte) (i >> 0 * 8);
		return dest;
	}




	/**********************************************************************
	Breaks int into little endian bytes.

	<p><b>Details:</b>  This method breaks an <code>int</code> into 4 <code>byte</code>s and stores them in the given array, at the given index, using little endian byte ordering.</p>

	@param i the int
	@param dest the destination array
	@param offset the array index
	@return dest
	**********************************************************************/

	public static final byte[] toBytesLe (int i, byte[] dest, int offset)
	{
		dest [offset + 3] = (byte) (i >> 3 * 8);
		dest [offset + 2] = (byte) (i >> 2 * 8);
		dest [offset + 1] = (byte) (i >> 1 * 8);
		dest [offset + 0] = (byte) (i >> 0 * 8);
		return dest;
	}




	/**********************************************************************
	Breaks short into little endian bytes.

	<p><b>Details:</b>  This method breaks a <code>short</code> into 2 <code>byte</code>s and stores them in the given array, at the given index, using little endian byte ordering.</p>

	@param s the short
	@param dest the destination array
	@param offset the array index
	@return dest
	**********************************************************************/

	public static final byte[] toBytesLe (short s, byte[] dest, int offset)
	{
		dest [offset + 1] = (byte) (s >> 1 * 8);
		dest [offset + 0] = (byte) (s >> 0 * 8);
		return dest;
	}




	////////////////////////
	//                    //
	//  endian-reversals  //
	//                    //
	////////////////////////



	/**********************************************************************
	Reverses endianness of int.

	@param i the int
	@return the reversed int
	**********************************************************************/

	public static final int reverseEndian (int i)
	{
		return
			(i & 0xff000000) >>> 24
		|	(i & 0x00ff0000) >>   8
		|	(i & 0x0000ff00) <<   8
		|	(i & 0x000000ff) <<  24;
	}



}



